import pyximport; pyximport.install()

import os
import unittest
import sure
from mock import patch

from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.models.exchange import Exchange, Position
from gryphon.lib.models.transaction import Transaction

import logging
logger = logging.getLogger(__name__)

class TestTransaction():
    def mockExchange(self, name=None):
        exchange_name = "TEST"
        if name:
            exchange_name += ("_" + name)
        exchange = Exchange(exchange_name)
        exchange.currency = "USD"
        return exchange

    def setUp(self):
        amount = Money(1, "BTC")

        self.source_exchange = self.mockExchange("SOURCE")
        self.target_exchange = self.mockExchange("TARGET")

        self.deposit = Transaction(Transaction.DEPOSIT, Transaction.IN_TRANSIT, amount, self.target_exchange, {})
        self.withdrawal = Transaction(Transaction.WITHDRAWL, Transaction.IN_TRANSIT, amount, self.source_exchange, {})

    def tearDown(self):
        pass

    def test_withdrawal(self):
        old_balance = self.source_exchange.balance['BTC']
        self.withdrawal.complete()
        new_balance = self.source_exchange.balance['BTC']
        balance_delta = new_balance - old_balance
        balance_delta.should.equal(Money.loads("BTC -1"))

    def test_deposit(self):
        old_balance = self.target_exchange.balance['BTC']
        self.deposit.complete()
        new_balance = self.target_exchange.balance['BTC']
        balance_delta = new_balance - old_balance
        balance_delta.should.equal(Money.loads("BTC 1"))

    def test_withdrawal_fee(self):
        self.withdrawal.fee = Money.loads("BTC 0.1")
        old_balance = self.source_exchange.balance['BTC']
        self.withdrawal.complete()
        new_balance = self.source_exchange.balance['BTC']
        balance_delta = new_balance - old_balance
        balance_delta.should.equal(Money.loads("BTC -1.1"))

    def test_deposit_fee(self):
        self.deposit.fee = Money.loads("BTC 0.1")
        old_balance = self.target_exchange.balance['BTC']
        self.deposit.complete()
        new_balance = self.target_exchange.balance['BTC']
        balance_delta = new_balance - old_balance
        balance_delta.should.equal(Money.loads("BTC 0.9"))

    def test_withdrawal_completed_cancel(self):
        self.withdrawal.fee = Money.loads("BTC 0.1")
        old_balance = self.source_exchange.balance['BTC']
        self.withdrawal.complete()
        self.withdrawal.cancel()
        new_balance = self.target_exchange.balance['BTC']
        new_balance.should.equal(old_balance)

    def test_deposit_completed_cancel(self):
        self.deposit.fee = Money.loads("BTC 0.1")
        old_balance = self.source_exchange.balance['BTC']
        self.deposit.complete()
        self.deposit.cancel()
        new_balance = self.target_exchange.balance['BTC']
        new_balance.should.equal(old_balance)

    def test_withdrawal_in_transit_cancel(self):
        old_balance = self.source_exchange.balance['BTC']
        self.withdrawal.cancel()
        self.withdrawal.transaction_status.should.equal(Transaction.CANCELED)
        new_balance = self.target_exchange.balance['BTC']
        new_balance.should.equal(old_balance)

    def test_double_complete(self):
        self.withdrawal.complete()
        self.withdrawal.complete.when.called_with().should.throw(ValueError)

    def test_double_cancel(self):
        self.withdrawal.cancel()
        self.withdrawal.cancel.when.called_with().should.throw(ValueError)



# -*- coding: utf-8 -*-
import base64
import datetime
import hashlib
import hmac
import json
import math
import os
import requests
import time
import urllib
import decimal
from cdecimal import *
from collections import OrderedDict
from delorean import Delorean, parse
from urllib import urlencode

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.base import *
from gryphon.lib.exchange.exceptions import *
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

logger = get_logger(__name__)


class CoinsetterExchange(Exchange):
    def __init__(self, session=None, use_cached_orderbook=False):
        super(CoinsetterExchange, self).__init__(session)
        self.name = u'COINSETTER'
        self.friendly_name = u'Coinsetter'
        self.base_url = 'https://api.coinsetter.com/v1'
        self.currency = "USD"
        self.fee = Decimal("0.002")
        self.market_order_fee = self.fee
        self.limit_order_fee = self.fee
        self.bid_string = "BUY"
        self.ask_string = "SELL"
        self.use_cached_orderbook = use_cached_orderbook



import json
from datetime import datetime

import numpy as np
import pandas as pd
from sqlalchemy.dialects.mysql import DATETIME
from sqlalchemy import Column, Integer, Numeric, Index, DateTime

from metric_types import get_metric_type_int
from base import AtlasZeroBase

metadata = AtlasZeroBase.metadata


class MetricException(Exception):
    pass


class Metric(AtlasZeroBase):
    __tablename__ = 'metric'
    __table_args__ = (Index('idx_metric_type_timestamp', 'metric_type', 'timestamp'), )

    metric_id = Column(Integer, primary_key=True)
    timestamp = Column(DATETIME(fsp=6), nullable=False)
    time_created = Column(DateTime, nullable=False)
    metric_type = Column(Integer, nullable=False)
    value = Column(Numeric(precision=20, scale=10))

    def __init__(self, metric_type, value, timestamp):
        self.time_created = datetime.utcnow()
        self.timestamp = timestamp

        self.metric_type = get_metric_type_int(metric_type)
        self.value = value

    def __unicode__(self):
        return unicode(repr(self))

    def __repr__(self):
        d = {
            'metric_id': self.metric_id,
            'timestamp': str(self.timestamp),
            'value': str(self.value),
            'metric_type': self.metric_type,
        }

        return json.dumps(d, ensure_ascii=False)

    @classmethod
    def get_as_pandas_by_series_id(cls, db, series_id):
        raw_series = db.query(Metric).filter(Metric.metric_type == series_id).all()

        pd_series = cls.convert_metric_series_to_pandas(raw_series)

        return pd_series

    @classmethod
    def convert_metric_series_to_pandas(cls, metric_series):
        metric_series = sorted(metric_series, key=lambda m: m.timestamp)

        index = [m.timestamp for m in metric_series]
        values = [float(m.value) if m.value is not None else np.nan for m in metric_series]

        return pd.Series(values, index=index)



from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts

class Order(object):
    __slots__ = ('status', 'exchange', 'order_id', 'price', 'volume', 'type')
    BID = Consts.BID
    ASK = Consts.ASK
    OPEN =u'OPEN'

    CREATED = u'CREATED'
    UNFILLED = u'UNFILLED'
    PARTIALLY_FILLED = u'PARTIALLY_FILLED'
    FILLED = u'FILLED'
    CANCELLED = u'CANCELLED'
    # we use this to flag our orders in an orderbook
    FLAGGED = u'FLAGGED'
    
    
    def  __init__(self, price, volume, exchange, type, order_id=None, status=None):
        self.status = status if status else Order.OPEN
        self.order_id = order_id
        self.price = price
        self.volume = volume
        self.exchange = exchange
        self.type = type

    def __str__(self):
        return (u'[EXCHANGE ORDER] - %s - %s -[%s @ %s] - Status:%s - Order ID:%s' %
            (self.type, self.exchange.name, self.volume, self.price, self.status, self.order_id))

    def __repr__(self):
        return "<" + unicode(self) + ">"

    def __eq__(self, other):
        return (self.order_id == other.order_id and 
            self.price == other.price and 
            self.volume == other.volume and 
            self.type == other.type and
            self.exchange == other.exchange)
            
    def __lt__(self, other):
        return self.price < other.price
    
    def apply_fee(self):
        if self.type == Order.ASK:
            self.price += (self.price * self.exchange.fee)
        elif self.type == Order.BID:
            self.price -= (self.price * self.exchange.fee)
    
    
    @staticmethod
    def scopy(order_to_copy):
        return Order(
            order_to_copy.price,
            order_to_copy.volume,
            order_to_copy.exchange, 
            order_to_copy.type,
            order_to_copy.order_id, 
            order_to_copy.status)

    @staticmethod
    def copy(order_to_copy):
        return Order(
            Money(order_to_copy.price.amount, order_to_copy.price.currency),
            Money(order_to_copy.volume.amount, order_to_copy.volume.currency),
            order_to_copy.exchange, 
            order_to_copy.type,
            order_to_copy.order_id, 
            order_to_copy.status)




import pyximport; pyximport.install()

import unittest
from decimal import Decimal

import logging
logger = logging.getLogger(__name__)

from gryphon.lib.exchange.sim_exchange import SimExchange
from backtesting.engine import Engine
from backtesting.orderflow_generator.regimes import RegimeBasedGenerator
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.money import Money

class TestEngine(unittest.TestCase):
    def setUp(self):
        self.exchange = SimExchange(engine=Engine(RegimeBasedGenerator(1)))
    def tearDown(self):
        pass

    """
        Instantiating SimExchange should create an engine as well
    """
    def test_create_engine(self):
        self.assertTrue(self.exchange.engine != None)

    """
        A single call to get_orderbook should increase the size of the 
        orderbook to one, and the next call should increase it to two.
    """
    def test_get_orderbook_creates_an_order(self):
        orderbook = self.exchange.get_order_book()

        orderbook_size = len(orderbook.get('bids')) + len (orderbook.get('asks'))

        self.assertTrue(orderbook_size == 1)
      
        new_orderbook = self.exchange.get_order_book()

        new_orderbook_size = len(new_orderbook.get('bids')) + len(new_orderbook.get('asks'))

        self.assertTrue(new_orderbook_size == 2)


    """
      With the default seed value, fill_order_book should leave the
      test exchange with 73 orders on its books. If this has changed
      unexpectedly, we should know about it. Based on Baseline config 1
    """
    def test_fill_order_book_behaviour_has_not_changed(self):
        self.exchange.fill_order_book()

        orderbook = self.exchange.get_order_book()

        bids = orderbook.get('bids')
        asks = orderbook.get('asks')
        self.assertTrue(len(bids) + len(asks) == 73)
   
 

    """
        If we set a bid at the same price as an ask, those two orders
        should be matched and executed by our exchange engine.

        This only works because we know that the first order that is 
        generated based on the current random seed.
    """
    def test_buy_order_should_match(self):
        orderbook = self.exchange.get_order_book()

        ask = orderbook.get('asks', [])[0]

        order_response = self.exchange.create_trade(Order.BID, ask.volume, ask.price)

        order_id = order_response.get('order_id')

        statuses = self.exchange.multi_order_status([order_id])

        # there should only be one status here
        self.assertTrue(len(statuses) == 1)

        for order_id in statuses.keys():
            self.assertTrue(statuses[order_id]['status'] == 'filled')


    """
        If we set a bid at a price much lower than the initial ask, it
        should not be matched or executed in our exchange engine
    """
    def test_buy_order_should_not_match(self):
        orderbook = self.exchange.get_order_book()

        ask = orderbook.get('asks', [])[0]

        order_response = self.exchange.create_trade(
            Order.BID, 
            ask.volume, 
            ask.price - 2,
        )

        order_id = order_response.get('order_id')

        statuses = self.exchange.multi_order_status([order_id])

        # there should only be one status here
        self.assertTrue(len(statuses) == 1)
        
        logger.error(statuses)
        
        for order_id in statuses.keys():
            self.assertTrue(statuses[order_id]['status'] == 'open')


    """
        If we buy bitcoins, our fiat account balance should change by the
        appropriate amount in our exchange engine.
    """
    def test_account_balance_changes_after_buy(self):
        balances = self.exchange.balance()

        orderbook = self.exchange.get_order_book()

        ask = orderbook.get('asks', [])[0]

        order_response = self.exchange.create_trade(
            Order.BID, 
            ask.volume, 
            ask.price,
        )

        new_balances = self.exchange.balance()

        self.assertTrue(balances['BTC'] < new_balances['BTC'])
        self.assertTrue(balances['USD'] > new_balances['USD'])

    
    """
        If we create many trades, these trades should be returned by the
        exchange engine from an exchange.open_orders() call
    """
    def test_open_orders(self):
        for i in range(0,5):
            order_response = self.exchange.create_trade(
                Order.BID, 
                Money(1, 'BTC'),
                Money('500', 'USD'),
            )

        open_orders = self.exchange.open_orders()

        self.assertTrue(len(open_orders) == 5)

   
    """
        Test that the order_details function (which is poorly named because
        it is actually querying trades the relate to an order, not just
        the details of that order), is properly working.
    """ 
    def test_order_details(self):
        # create an absurdly high bid
        order_response = self.exchange.create_trade(
            Order.BID, 
            Money(1, 'BTC'),
            Money('100000', 'USD'),
        )
        
        order_id = order_response.get('order_id')

        order_details = self.exchange.order_details(order_id)
        logger.debug(order_details)

        # there's nothing in the order book yet so this should not have
        # matched
        self.assertTrue(len(order_details['trades']) == 0)
        
        self.exchange.fill_order_book()
        orderbook = self.exchange.get_order_book()
        
        new_order_details = self.exchange.order_details(order_id)

        # the high bid should have matched with new ask in the orderbook
        # so there should be trades in this order details
        self.assertTrue(len(new_order_details['trades']) > 0)


    """
        If we create an order and then cancel it, it should no longer
        appear in the results of an open_orders call.
    """
    def test_cancel_order(self):
        order_response = self.exchange.create_trade(
            Order.BID, 
            Money('1', 'BTC'),
            Money('100000', 'USD'),
        )

        order_id = order_response.get('order_id')

        open_orders = self.exchange.open_orders()

        self.assertTrue(len(open_orders) == 1)

        self.exchange.cancel_order(order_id)

        open_orders = self.exchange.open_orders()

        self.assertTrue(len(open_orders) == 0)


    """
        If we create many orders, they should all be present in the results
        of a exchange.multi_order_status call
    """
    def test_multi_order_status_req(self):
        order_ids = []

        for i in range(0,5):
            order_response = self.exchange.create_trade(
                Order.BID, 
                Money('1', 'BTC'),
                Money('1000', 'USD'),
            )

            order_ids.append(order_response['order_id'])

        order_statuses = self.exchange.multi_order_status(order_ids)

        self.assertTrue(len(order_statuses) == 5)

        for order_id in order_statuses.keys():
            order_status = order_statuses[order_id]['status']
            self.assertTrue(order_status == 'open')
    




# -*- coding: utf-8 -*-
import time
import os
import datetime
import requests
import hmac
import urllib
import hashlib
import base64
import json
import math
from collections import defaultdict, OrderedDict
import copy
from delorean import Delorean, parse, epoch

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money
from exchange_order import Order
from base import *
from exceptions import *
from gryphon.lib.models.exchange import Balance

from cdecimal import *
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)

class CaVirtexExchange(Exchange):
    def __init__(self, session=None, currency=u"CAD", use_cached_orderbook=False):
        super(CaVirtexExchange, self).__init__(session)
        self.name = u'CAVIRTEX'
        self.friendly_name = u'CaVirtex'
        self.base_url = 'https://www.cavirtex.com/api2'
        self.currency = currency
        self.fee = Decimal("0.00") # special 2-month zero-fee deal
        self.market_order_fee = self.fee
        self.limit_order_fee = self.fee
        self.withdrawal_fee = Money("0.001", "BTC")
        self.bid_string = "buy"
        self.ask_string = "sell"
        self.use_cached_orderbook = use_cached_orderbook



"""
ExchangeAPIWrapper base class.

These classes make heavy use of the requests-futures library, which allows us to begin
API requests and then continue execution until we require the result of the request.
This can be a substantial performance increase when doing many requests in a single
thread. To this end most of the endpoints that we support are split into a _req and
_resp function. The first begins the request, and the second blocks on the response.
For each endpoint there is also a function which simply calls both parts in serial. You
can safely use these when performance is not crucial.
"""

from collections import defaultdict
import errno
import functools
import inspect
import json
import os
from socket import error as SocketError

from cdecimal import Decimal
from delorean import Delorean, epoch
import termcolor as tc
import requests
from requests_futures.sessions import FuturesSession

from gryphon.lib.configurable_object import ConfigurableObject
from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_order import Order
from gryphon.lib.logger import get_logger
from gryphon.lib.metrics import quote as quote_lib
from gryphon.lib.money import Money
from gryphon.lib.session import get_a_redis_connection

logger = get_logger(__name__)


class ExchangeAPIWrapper(ConfigurableObject):
    CACHED_ORDERBOOK = 'CACHED_ORDERBOOK'
    LOG_LINE_LIMIT = 100

    def __init__(self, session=None, configuration=None):
        if session:
            self.session = session
        else:
            self.session = FuturesSession(max_workers=10)

        self.withdrawal_fee = Money('0', 'BTC')
        self.btc_credit_limit = Money('0', 'BTC')
        self.price_decimal_precision = 2
        self.volume_decimal_precision = None
        self.volume_currency = 'BTC'

        # Configurable properties with defaults.
        self.fee = Decimal('0.0005')
        self.market_order_fee = self.fee
        self.limit_order_fee = self.fee
        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.min_order_size = Money('0.0001', 'USD')
        self.max_tick_speed = 2
        self.use_cached_orderbook = False

    def configure(self, configuration):
        """
        Initialize fees, balance tolerances, and whether to use cached orderbooks.

        There's an annoying leaking of responsibilities in this function right now.
        It's really the ExchangeCoordinator in gryphon.execution that should know the
        destination for requests, this class should just implement the Exchange API.
        """
        platform_configuration = configuration['platform']
        exchange_configuration = None

        if ('exchanges' in configuration
                and self.name.lower() in configuration['exchanges']):
            exchange_configuration = configuration['exchanges'][self.name.lower()]

        if exchange_configuration is not None and 'emerald' in exchange_configuration:
            # Exchange-specific configuraton overrides the platform's use of emerald
            # or not.
            self.use_cached_orderbook = exchange_configuration['emerald']
        elif 'emerald' in platform_configuration:
            self.use_cached_orderbook = platform_configuration['emerald']

        if not exchange_configuration:
            return

        self.init_configurable('market_order_fee', exchange_configuration)
        self.init_configurable('limit_order_fee', exchange_configuration)
        self.init_configurable('fiat_balance_tolerance', exchange_configuration)
        self.init_configurable('volume_balance_tolerance', exchange_configuration)
        self.init_configurable('max_tick_speed', exchange_configuration)
        self.init_configurable('min_order_size', exchange_configuration)

    def place_order(self, mode, volume, price=None, order_type=order_types.LIMIT_ORDER):
        """
        Place an order on the exchange.
        """
        req = self.place_order_req(mode, volume, price, order_type)
        return self.place_order_resp(req)
 
    def place_order_req(self, mode, volume, price=None, order_type=order_types.LIMIT_ORDER):
        raise NotImplementedError

    def place_order_resp(self, req):
        raise NotImplementedError

    def market_order(self, mode, volume):
        """
        Place a market order on the exchange.
        """
        return self.place_order(mode, volume, order_type=order_types.MARKET_ORDER)
    
    def market_order_req(self, mode, volume):
        raise NotImplementedError

    def market_order_resp(self, req):
        raise NotImplementedError

    def limit_order(self, mode, volume, price):
        """
        Place a limit order on the exchange.
        """
        return self.place_order(mode, volume, price)

    def limit_order_req(self, mode, volume, price):
        raise NotImplementedError

    def limit_order_resp(self, req):
        raise NotImplementedError

    def get_open_orders(self):
        req = self.get_open_orders_req()
        return self.get_open_orders_resp(req)

    def get_open_orders_req(self):
        raise NotImplementedError

    def get_open_orders_resp(self, req):
        raise NotImplementedError

    def cancel_order(self, order_id):
        """
        Cancel an open order.
        """
        req = self.cancel_order_req(order_id)
        return self.cancel_order_resp(req)

    def cancel_order_req(self, order_id):
        raise NotImplementedError

    def cancel_order_resp(self, req):
        raise NotImplementedError

    def cancel_all_open_orders(self):
        """
        Cancel all open orders. Some exchanges provide a special endpoint for this, in
        which case the subclass should override this function.
        """
        for order in self.get_open_orders():
            self.cancel_order(order['id'])

    def get_order_details(self, order_id):
        """
        Get all the relevant information about any order, open or close. Exchanges have
        divergent behaviour on this action, some don't give any information about
        cancelled orders, some keep records on cancelled orders for a limited time, and
        some keep records forever.
        """
        req = self.get_order_details_req(order_id)
        return self.get_order_details_resp(req)

    def get_order_details_req(self, order_id):
        raise NotImplementedError

    def get_order_details_resp(self, req):
        raise NotImplementedError

    def get_multi_order_details(self, order_ids):
        """
        Get information for several orders at once. Some exchanges provide an endpoint
        that gives information about multiple orders at once, and this is way quicker
        than doing independent requests for each order.
        """
        req = self.get_multi_order_details_req(order_ids)
        return self.get_multi_order_details_resp(req)

    def get_multi_order_details_req(self, order_ids):
        raise NotImplementedError

    def get_multi_order_details_resp(self, req):
        raise NotImplementedError
   
    def get_balance(self):
        """
        Get our account balance on the exchange.
        """
        req = self.get_balance_req()
        return self.get_balance_resp(req)
 
    def get_balance_req(self):
        raise NotImplementedError

    def get_balance_resp(self, req):
        raise NotImplementedError

    def withdraw(self):
        """
        Withdraw part of our balance from this exchange.
        """
        pass

    def withdraw_req(self):
        raise NotImplementedError

    def withdraw_resp(self, req):
        raise NotImplementedError

    def get_order_audit_data(self):
        """
        This function gets a list of recent orders on our account mapped to the amount
        of that order that was filled. This function is used heavily in auditing.
        """
        raise NotImplementedError

    def get_ticker(self, verify=True):
        """
        Get the current ticker, which usually has a summary of recent activity on the
        market.
        """
        req = self.get_ticker_req(verify=verify)
        return self.get_ticker_resp(req)

    def get_ticker_req(self, verify=True):
        raise NotImplementedError

    def get_ticker_resp(self, req):
        raise NotImplementedError

    def get_orderbook(self, volume_limit=None, verify=True):
        """
        Get the current orderbook.
        """
        req = self.get_orderbook_req(verify)
        return self.get_orderbook_resp(req, volume_limit)

    def get_orderbook_req(self, verify=True):
        if self.use_cached_orderbook:
            return ExchangeAPIWrapper.CACHED_ORDERBOOK

        return self._get_orderbook_from_api_req(verify=verify)

    def get_orderbook_resp(self, req, volume_limit=None):
        if req == ExchangeAPIWrapper.CACHED_ORDERBOOK:
            return self._get_orderbook_from_cache(volume_limit)
        else:
            raw_orderbook = self._get_orderbook_from_api_resp(req)

            parsed_orderbook = self.parse_orderbook(raw_orderbook, volume_limit)
            parsed_orderbook['time_parsed'] = Delorean().epoch

            return parsed_orderbook

    def get_price_quote(self, mode, volume):
        """
        Returns an expected total cost to fill a given order, as well as the price the
        order should be entered at to clear the desired volume if it is a limit order.
        """
        req = self.get_price_quote_req()
        return self.get_price_quote_resp(req, mode, volume)

    def get_price_quote_req(self):
        return [self.get_open_orders_req(), self.get_orderbook_req()]

    def get_price_quote_resp(self, reqs, mode, volume):
        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Volume must be %s' % self.volume_currency)

        open_orders = self.get_open_orders_resp(reqs[0])

        # We need to ignore orders we currently have on the books as most exchanges
        # don't allow self-trading.
        open_volume = sum([o['volume_remaining'] for o in open_orders])
        orderbook = self.get_orderbook_resp(reqs[1], volume + open_volume)

        orderbook = self.remove_orders_from_orderbook(orderbook, open_orders)

        return quote_lib.price_quote_from_orderbook(orderbook, mode, volume)

    def remove_orders_from_orderbook(self, orderbook, open_orders, only_flag=False):
        """
        Remove our own orders from an orderbook.
        """
        open_volume_by_price = defaultdict(int)

        for o in open_orders:
            open_volume_by_price[o['price'].amount] += o['volume_remaining']

        for order in orderbook['bids'] + orderbook['asks']:
            open_volume = open_volume_by_price.get(
                order.price.amount, Money('0', 'BTC'),
            )

            if open_volume > 0:
                volume_to_remove = min(open_volume, order.volume)

                if only_flag:
                    order.status = Order.FLAGGED
                else:
                    order.volume -= volume_to_remove

                open_volume -= volume_to_remove

        # Remove 0-volume orders.
        orderbook['bids'] = [o for o in orderbook['bids'] if o.volume > 0]
        orderbook['asks'] = [o for o in orderbook['asks'] if o.volume > 0]

        return orderbook

    @property
    def _orderbook_sort_key(self):
        return lambda order: float(order[0])

    def parse_order(self, order):
        price = Money(order[0], self.currency)
        volume = Money(order[1], self.volume_currency)

        return price, volume

    def parse_cached_order(self, order):
        price = Money(order[0], self.currency)
        volume = Money(order[1], self.volume_currency)
        return (price, volume)

    def parse_any_order(self, order, is_cached_order=False):
        if is_cached_order:
            price, volume = self.parse_cached_order(order)
        else:
            price, volume = self.parse_order(order)

        return price, volume

    def parse_orderbook(self, raw_orderbook, volume_limit=None, price_limit=None, cached_orders=False):
        """
        Returns a dictionary containing a sorted list of asks and a sorted list of bids.
        """

        if cached_orders:
            raw_bids = raw_orderbook['bids']
            raw_asks = raw_orderbook['asks']
            sort_key = lambda o: float(o[0])
        else:
            raw_bids = self._get_raw_bids(raw_orderbook)
            raw_asks = self._get_raw_asks(raw_orderbook)
            sort_key = self._orderbook_sort_key

        raw_bids.sort(key=sort_key, reverse=True)
        raw_asks.sort(key=sort_key)

        bids = []
        asks = []

        total_volume = Money(0, self.volume_currency)
        top_bid_price, _ = self.parse_any_order(raw_bids[0], cached_orders)

        for bid in raw_bids:
            bid_price, bid_volume = self.parse_any_order(bid, cached_orders)

            if price_limit:
                bid_threshold = top_bid_price - price_limit

                if bid_price < bid_threshold:
                    break

            bids.append(Order(bid_price, bid_volume, self, Order.BID))

            total_volume += bid_volume

            if volume_limit and total_volume > volume_limit:
                break

        total_volume = Money('0', self.volume_currency)
        top_ask_price, __ = self.parse_any_order(raw_asks[0], cached_orders)

        for ask in raw_asks:
            ask_price, ask_volume = self.parse_any_order(ask, cached_orders)

            if price_limit:
                ask_threshold = top_ask_price + price_limit

                if ask_price > ask_threshold:
                    break

            asks.append(Order(ask_price, ask_volume, self, Order.ASK))

            total_volume += ask_volume

            if volume_limit and total_volume > volume_limit:
                break

        return {'bids': bids, 'asks': asks}


    # Request methods. All requests to exchange APIs are filtered through these
    # functions which make heavy use of the request-futures library to minimize
    # blocking on Exchange API calls.

    def req(self, req_method, url, **kwargs):
        """
        Place a request to the exchange api and return us a request-future object so
        that our code-path can continue execution until we need the result of the
        request.
      
        This function handles the interface with the request-futures library as well as
        dispatches to the auth_request method if we're making an authenticated call like
        to place an order or get our balance.
        """

        # If the exchange has a base_url set for it's API we can send in only the
        # path to the API endpoint we're trying to reach.
        if url[:4] != 'http':
            url = self.base_url + url

        try:
            no_auth = kwargs['no_auth']
            del kwargs['no_auth']
        except KeyError:
            no_auth = False

        if not no_auth:
            self.auth_request(req_method, url, kwargs)

        req_method = req_method.lower()

        if req_method == 'post':
            future = self.session.post(url, **kwargs)
        elif req_method == 'get':
            future = self.session.get(url, **kwargs)
        elif req_method == 'delete':
            future = self.session.delete(url, **kwargs)
        else:
            raise ValueError('%s method not supported' % req_method)

        return future

    def resp(self, req):
        """
        Get the response from a requests-futures object.
        """

        # Get the function name that called us.
        try:
            stack = inspect.stack()
            api_method = stack[1][3]

            if api_method == 'resp':
                api_method = stack[2][3]

            api_method = api_method.replace('_resp', '')
        except IndexError:
            api_method = 'unknown'

        response = None

        try:
            response = req.result()
            log_line = response.text

            if len(log_line) > ExchangeAPIWrapper.LOG_LINE_LIMIT:
                log_line = log_line[: ExchangeAPIWrapper.LOG_LINE_LIMIT]
                log_line += '...'

            logger.debug(u'[%s] API %s - %s' % (
                self.friendly_name,
                api_method,
                log_line,
            ))

            data = response.json(parse_float=Decimal)
        except ValueError:  # This includes JSONDecodeError.
            raise exceptions.ExchangeAPIFailureException(self, response)
        except (requests.exceptions.ConnectionError,
                    requests.exceptions.ChunkedEncodingError):
            raise exceptions.ExchangeAPIFailureException(self)
        except SocketError as e:
            if e.errno == errno.ECONNRESET:
                raise exceptions.ExchangeAPIFailureException(self)
            else:
                raise e

        return data

    def auth_request(req_method, url, request_args):
        """
        This function handles all calls to authenticated endpoints in the exchange API,
        e.g. getting balances or placing orders, in contrast to public methods like
        getting the current orderbook.
        """
        raise NotImplementedError

    # Helper functions and private methods.

    def _get_orderbook_from_api_req(self, verify=True):
        """
        Get the current raw (unparsed) orderbook directly from the exchange API.
        """
        raise NotImplementedError

    def _get_orderbook_from_api_resp(self, req):
        return self.resp(req)

    def _get_raw_bids(self, raw_orderbook):
        """
        This function parses the bids out of the raw orderbook. This is a sane default
        function that works for most exchanges, the rest need to overload this function
        in their implementation.
        """
        return raw_orderbook['bids']

    def _get_raw_asks(self, raw_orderbook):
        """
        Sibling to _get_raw_bids.
        """
        return raw_orderbook['asks']

    def _get_orderbook_from_cache(self, volume_limit=None):
        """
        Get the current orderbook for the exchange from our local cache.
        """

        key = '%s_orderbook' % self.name.lower()
        raw_redis_result = self.redis.get(key)

        try:
            redis_result = json.loads(raw_redis_result)

            orderbook_age = Delorean().epoch - float(redis_result['timestamp'])

            if orderbook_age >= 10:
                raise exceptions.CachedOrderbookFailure(
                    self,
                    'Cached orderbook more than 10 seconds old',
                )

            time_fetched = redis_result['timestamp']
            orderbook = redis_result[self.name]

            parsed_orderbook = self.parse_orderbook(
                orderbook,
                volume_limit,
                cached_orders=True,
            )

            parsed_orderbook['time_fetched'] = time_fetched

            return parsed_orderbook
        except exceptions.CachedOrderbookFailure:
            raise
        except Exception as e:
            raise exceptions.CachedOrderbookFailure(
                self,
                'Unable to parse cached orderbook',
            )

    def _load_env(self, key):
        return str(os.environ[key])

    def _order_mode_from_const(self, mode):
        if mode == Consts.BID:
            return self.bid_string
        elif mode == Consts.ASK:
            return self.ask_string
        else:
            raise ValueError('mode must be a Const')

    def _order_mode_to_const(self, mode):
        if mode == self.bid_string:
            return Consts.BID
        elif mode == self.ask_string:
            return Consts.ASK
        else:
            raise ValueError(
                'mode must be one of %s/%s' % (self.bid_string, self.ask_string),
            )

    @property
    def redis(self):
        if hasattr(self, '_redis'):
            return self._redis
        else:
            self._redis = get_a_redis_connection()
            return self._redis

    def maker_taker_fee_difference(self):
        """
        Simple helper function to give the fee difference between market and limit
        orders on this exchange. Defaults to zero (no difference).
        """
        if self.market_order_fee != None and self.limit_order_fee != None:
            return self.market_order_fee - self.limit_order_fee
        else:
            return Decimal('0')

    def exchange_account_db_object(self, db):
        """
        Just a helper function to get our database representation of the current
        account.
        """
        return exchange_factory.make_exchange_data_from_key(self.name, db)

    def process_db_balance_for_audit(self, db_balance):
        """
        Exchange trading engines have different characteristics when it comes to
        decimal precision and rounding behaviour. This function massages our database
        record of our balance on this exchange so that it can be compared to the
        exchange API's output.
        """
        return db_balance

    def pre_audit(self, exchange_data):
        """
        Some exchanges have extra actions that need to be taken before an audit.
        """
        pass

    def fiat_deposit_fee(self, deposit_amount):
        return Money('0', self.currency)

    def fiat_withdrawal_fee(self, deposit_amount):
        return Money('0', self.currency)

    @property
    def current_deposit_address(self):
        deposit_address_env_key = '%s_DEPOSIT_ADDRESS' % self.name
        return self._load_env(deposit_address_env_key)

    def get_new_deposit_address(self):
        """
        TODO: This method is unimplemented everywhere, but some exchanges do supply this
        functionality in their API, so it would be a good thing add in the future.
        """
        raise NotImplementedError

    def deposit_address_req(self):
        raise NotImplementedError

    def deposit_address_resp(self, req):
        raise NotImplementedError

    def withdraw_crypto(self, address, volume):
        """
        Withdraw a cryptocurrency amount to an external address.
        """
        req = self.withdraw_crypto_req(address, volume)
        return self.withdraw_crypto_resp(req)

    def withdraw_crypto_req(self, address, volume):
        raise NotImplementedError

    def withdraw_crypto_resp(self, req):
        raise NotImplementedError




import pyximport; pyximport.install()

from gryphon.lib.exchange.bitstamp_btc_usd import BitstampBTCUSDExchange
from gryphon.tests.exceptional.exchange_wrappers.live_orders import LiveOrdersTest


class TestBitstampBTCUSDLiveOrders(LiveOrdersTest):
    def __init__(self):
        # BitstampBTCUSD has a $5 minimum order size.
        self.order1_price_amount = '5'
        self.order2_price_amount = '6'
        self.sleep_time = 1

    def setUp(self):
        self.exchange = BitstampBTCUSDExchange()



"""
Simple function to get the quote for a desired market order from an orderbook. This is
useful in a variety of ways and places.
"""


from gryphon.lib.money import Money
from gryphon.lib.exchange.consts import Consts


def price_quote_from_orderbook(order_book, mode, volume):
    if mode == Consts.BID:
        orders = order_book.get('asks', [])
    elif mode == Consts.ASK:
        orders = order_book.get('bids', [])
    else:
        raise ValueError('mode must be one of ask/bid')

    if not isinstance(volume, Money):
        raise ValueError('Volume must be a Money() object')

    if volume.currency != orders[0].volume.currency:
        raise ValueError('Volume currency does not match orderbook currency! %s != %s' % (
            volume.currency,
            orders[0].volume.currency,
        ))

    if not orders:
        raise Exception('no orders on one side of the book.')

    price = 0
    volume_remaining = volume

    for order in orders:
        volume_from_this_order = order.volume

        if order.volume > volume_remaining:
            volume_from_this_order = volume_remaining

        volume_remaining -= volume_from_this_order
        price += (volume_from_this_order.amount * order.price)

        if volume_remaining <= 0:
            break

    last_order = order  # Because the loop broke.

    if volume_remaining > 0:
        raise Exception('not enough liquidity for a %s %s' % (volume, mode))

    response = {
        'total_price': price,
        'price_for_order': last_order.price,
    }

    return response




def humanize_seconds(seconds):
    """Take n seconds and return a human readable string (1h 2m 13s)"""
    m, s = divmod(seconds, 60)
    h, m = divmod(m, 60)
    chunks = []

    # 2nd condition is because we want 0 seconds to be represented as "0s" not ""
    if s or seconds == 0:
        seconds_chunk = "%ds" % s
        chunks.insert(0, seconds_chunk)
    if m:
        minutes_chunk = "%dm" % m
        chunks.insert(0, minutes_chunk)
    if h:
        hours_chunk = "%dh" % h
        chunks.insert(0, hours_chunk)

    return " ".join(chunks)



"""
A super simple strategy that looks for arbitrage between Gemini and Coinbase's BTCUSD
orderbooks, and executes trades to take advantage of opportunities.

This strategy is meant to be an example, and is not fleshed-out enough to be a long-term
viable strategy.
"""

from gryphon.execution.strategies.base import Strategy
from gryphon.lib import arbitrage as arb
from gryphon.lib.exchange.consts import Consts


class SuperSimpleArb(Strategy):
    def tick(self, open_orders):
        cross = arb.detect_directional_cross(
            self.harness.gemini_btc_usd.get_orderbook(),
            self.harness.coinbase_btc_usd.get_orderbook(),
        )

        executable_volume = arb.get_executable_volume(
            cross,
            self.harness.coinbase_btc_usd.get_balance(),
            self.harness.gemini_btc_usd.get_balance(),
        )

        if cross and executable_volume:
            self.harness.gemini_btc_usd.market_order(executable_volume, Consts.BID)
            self.harness.coinbase_btc_usd.market_order(executable_volume, Consts.ASK)




# -*- coding: utf-8 -*-
from delorean import Delorean
import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
from gryphon.dashboards.util import assets as assets_helper
from gryphon.lib import assets
import gryphon.lib.gryphonfury.fees as gryphon_fees
import gryphon.lib.gryphonfury.revenue as revenue_lib
from gryphon.lib.logger import get_logger
from gryphon.lib.money import Money

logger = get_logger(__name__)


class AssetsHandler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self):
        start_time, end_time = self.get_start_time_and_end_time()

        # Handle end_time in the future by only loading the series up to today
        now = Delorean()

        if now.datetime < end_time:
            now.truncate('day')
            end_time = now.datetime

        assets_series = assets_helper.get_assets_series(
            self.trading_db,
            start_time,
            end_time,
        )

        liabilities_series = assets_helper.get_liabilities_series(
            self.trading_db,
            start_time,
            end_time,
        )

        net_assets_series = assets_helper.calculate_net_assets_series(
            assets_series,
            liabilities_series,
        )

        starting_balance = net_assets_series[0][1]
        starting_usd_balance = starting_balance.total_usd_value(date=start_time)

        final_balance = net_assets_series[-1][1]
        final_usd_balance = final_balance.total_usd_value(date=end_time)

        change = final_usd_balance - starting_usd_balance

        burn_transactions = assets.get_burn_transactions(
            self.trading_db,
            start_time,
            end_time,
        )

        burn = Money('0', 'USD')

        # It's important to use 'is not' here as None indicates that the burn-
        # tracking feature is not set up.
        if burn_transactions is not None:

            for burn_transaction in burn_transactions:
                burn_position = burn_transaction.position

                burn += burn_position.total_usd_value(
                    date=burn_transaction.time_completed,
                )

        revenue, trading_fees, profit = revenue_lib.fast_revenue_fees_profit_in_period(
            self.trading_db,
            start_time,
            end_time,
        )

        wire_fees = gryphon_fees.get_all_wire_fees_in_period_in_usd(
            self.trading_db,
            start_time,
            end_time,
        )

        forex_flux = assets.calculate_forex_flux(
            starting_balance,
            final_balance,
            start_time,
            end_time,
        )

        expected_change = revenue - trading_fees - wire_fees - burn + forex_flux

        discrepancy = change - expected_change

        usd_net_assets_series = assets_helper.convert_balance_series_to_usd(
            net_assets_series,
        )

        flot_usd_net_assets_series = assets_helper.process_usd_series_for_flot(
            usd_net_assets_series,
        )

        usd_assets_series = assets_helper.convert_balance_series_to_usd(assets_series)
        flot_usd_assets_series = assets_helper.process_usd_series_for_flot(
            usd_assets_series,
        )

        usd_liabilities_series = assets_helper.convert_balance_series_to_usd(
            liabilities_series,
        )

        flot_usd_liabilities_series = assets_helper.process_usd_series_for_flot(
            usd_liabilities_series,
        )

        self.render_template(
            'assets.html',
            args={
                'flot_usd_net_assets_series': flot_usd_net_assets_series,
                'flot_usd_assets_series': flot_usd_assets_series,
                'flot_usd_liabilities_series': flot_usd_liabilities_series,
                'starting_usd_balance': starting_usd_balance,
                'final_usd_balance': final_usd_balance,
                'change': change,
                'burn': burn,
                'burn_transactions': burn_transactions,
                'revenue': revenue,
                'trading_fees': trading_fees,
                'wire_fees': wire_fees,
                'expected_change': expected_change,
                'discrepancy': discrepancy,
                'forex_flux': forex_flux,
            },
        )



from gryphon.data_service.auditors.coinbase_orderbook_auditor import CoinbaseOrderbookAuditor


class CoinbaseCADOrderbookAuditor(CoinbaseOrderbookAuditor):
    def __init__(self):
        super(CoinbaseCADOrderbookAuditor, self).__init__()
        self.exchange_name = 'COINBASE_CAD'
        product_id = 'BTC-CAD'
        self.orderbook_url = 'https://api.exchange.coinbase.com/products/%s/book?level=3' % product_id



"""
A simple strategy that looks for arbitrage between two exchange pairs' orderbooks. The
same tick logic as SuperSimpleArb, but with configurable buy and sell exchanges.

This strategy is meant to be an example, and is not fleshed-out enough to be a long-term
viable strategy. If you do use it, make sure that the exchange fees are configured
properly for both exchanges, or you will end up making seemingly-profitable trades, but
net losing money to fees.

Currently this strategy defaults to using gemini btc-usd and coinbase btc-usd as it's
buy and sell exchanges respectively unless this is overridden by other configuration.
"""

from gryphon.execution.strategies.base import Strategy
from gryphon.lib import arbitrage as arb
from gryphon.lib.exchange.consts import Consts


class SimpleArb(Strategy):
    def __init__(self, db, harness, strategy_configuration):
        super(SimpleArb, self).__init__(db, harness)

        # Configurable properties with defaults.
        self.buy_exchange = 'gemini_btc_usd'
        self.sell_exchange = 'coinbase_btc_usd'

        self.configure(strategy_configuration)

    def configure(self, strategy_configuration):
        super(SimpleArb, self).configure(strategy_configuration)

        self.init_configurable('buy_exchange', strategy_configuration)
        self.init_configurable('sell_exchange', strategy_configuration)

        self.buy_ex = self.harness.exchange_from_key(self.buy_exchange)
        self.sell_ex = self.harness.exchange_from_key(self.sell_exchange)

        self.target_exchanges = [self.buy_ex, self.sell_ex]

    def tick(self, open_orders):
        cross = arb.detect_directional_cross(
            self.buy_ex.get_orderbook(),
            self.sell_ex.get_orderbook(),
        )

        executable_volume = arb.get_executable_volume(
            cross,
            self.sell_ex.get_balance(),
            self.buy_ex.get_balance(),
        )

        if cross and executable_volume:
            self.buy_ex.market_order(executable_volume, Consts.BID)
            self.sell_ex.market_order(executable_volume, Consts.ASK)




import pyximport; pyximport.install()

import os
import unittest
import mock
from delorean import Delorean

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.money import Money

from gryphon.lib.models.order import Order

import logging
logger = logging.getLogger(__name__)

class TestOrder():

    def setUp(self):
        self.exchange = mock.Mock()
        self.order = Order('Test', Order.BID, Money("1", 'BTC'), Money("500", 'USD'), self.exchange, None)
        self.trade = {'btc': Money("0.1", 'BTC'),
            'trade_id': None,
            'fiat': Money("50", 'USD'),
            'fee': Money("1", 'USD'),
            'time': Delorean().epoch}
        self.order_details = {'btc_total': Money("0.1", 'BTC'),
            'trades': [self.trade],
            'type': Consts.BID,
            'fee': Money(0, 'USD'),
            'fiat_total': Money("50", 'USD')
            }
        pass

    def test_partially_eaten(self):
        positions, __ = self.order.was_partially_eaten(self.order_details)
        logger.debug(positions)
        btc_change = positions['BTC']
        fiat_change = positions['USD']
        btc_change.should.equal(Money("0.1", 'BTC'))
        fiat_change.should.equal(Money("-51", 'USD'))

        self.order.status.should.equal(Order.OPEN)
        self.order.trades.should.have.length_of(1)
        self.order.trades[0].volume.should.equal(Money("0.1", 'BTC'))

        self.order.volume_filled.should.equal(Money("0.1", 'BTC'))
        self.order.volume_remaining.should.equal(Money("0.9", 'BTC'))

    def test_partially_eaten_then_eaten(self):
        positions, __ = self.order.was_partially_eaten(self.order_details)
        btc_change = positions['BTC']
        fiat_change = positions['USD']
        btc_change.should.equal(Money("0.1", 'BTC'))
        fiat_change.should.equal(Money("-51", 'USD'))

        # order got cancelled
        positions, __ = self.order.was_eaten(self.order_details)
        btc_change = positions['BTC']
        fiat_change = positions['USD']

        # position should not change
        btc_change.should.equal(Money("0", 'BTC'))
        fiat_change.should.equal(Money("0", 'USD'))

    def test_multiple_partial_eats(self):
        order_details = self.order_details
        for i in range(3):
            positions, __ = self.order.was_partially_eaten(order_details)
            btc_change = positions['BTC']
            fiat_change = positions['USD']
            btc_change.should.equal(Money("0.1", 'BTC'))
            fiat_change.should.equal(Money("-51", 'USD'))
            order_details['trades'].append(self.trade)
            order_details['btc_total'] += Money("0.1", 'BTC')
            order_details['fiat_total'] += Money("50", 'USD')


        # order gets cancelled before we have a chance to "partially eat" the last change
        positions, __ = self.order.was_eaten(self.order_details)
        btc_change = positions['BTC']
        fiat_change = positions['USD']

        # position should reflect the one last change
        btc_change.should.equal(Money("0.1", 'BTC'))
        fiat_change.should.equal(Money("-51", 'USD'))

    def tearDown(self):
        pass

if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-
import json
import os

import praw
from gryphon.lib.session import get_a_redis_connection

# agent needs to be unique according to reddits api rules.
app_name = 'breaking_news_redditor_12809372502418712398'
story_list_key = 'news_already_seen'

def get_reddit_breaking_news(subreddit, upvote_threshold=500):
    try:
        r = praw.Reddit(user_agent=app_name)
        redis = get_a_redis_connection(creds=os.environ['NOODLE_DB_REDIS'])

        news_already_seen = json.loads(redis.get(story_list_key) or '[]')

        breaking_news = []

        submissions = r.get_subreddit(subreddit).get_hot(limit=100)

        for s in submissions:
            if s.score > upvote_threshold and s.url not in news_already_seen:
                news = {'title': s.title, 'url': s.url, 'comments_url': s.permalink}
                breaking_news.append(news)
                news_already_seen.append(s.url)

        redis.set(story_list_key, json.dumps(news_already_seen))
        return breaking_news
    finally:
        redis.connection_pool.disconnect()



import rq
import random
import signal
import os
from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


# patches RQ's work horse to not die on SIGTERM
class StrongWorker(rq.Worker):
    def main_work_horse(self, job):
        """This is the entry point of the newly spawned work horse."""
        # After fork()'ing, always assure we are generating random sequences
        # that are different from the worker.
        random.seed()

        # Always ignore Ctrl+C in the work horse, as it might abort the
        # currently running job.
        # The main worker catches the Ctrl+C and requests graceful shutdown
        # after the current work is done.  When cold shutdown is requested, it
        # kills the current job anyway.
        def nothing(signal, frame):
            return
        signal.signal(signal.SIGINT, nothing)
        signal.signal(signal.SIGTERM, nothing)

        self._is_horse = True
        self.log = logger

        success = self.perform_job(job)

        # os._exit() is the way to exit from childs after a fork(), in
        # constrast to the regular sys.exit()
        os._exit(int(not success))



from collections import defaultdict
import datetime
from decimal import Decimal

from delorean import Delorean
import sqlalchemy

from gryphon.lib.models.datum import Datum

def get_ticktime_series_for_exchange(db, exchange_name, start_time, end_time):
    exchange_name = exchange_name.upper()

    ticktime_series = []

    ticktimes = db\
        .query(Datum)\
        .filter_by(datum_type = '%s_TICK_TIME' % exchange_name)\
        .filter(Datum.time_created >= start_time)\
        .filter(Datum.time_created < end_time)\
        .all()

    if ticktimes:
        for tt in ticktimes:
            timestamp = Delorean(tt.time_created, "UTC").epoch * 1000
            datapoint = str(tt.numeric_value)

            ticktime_series.append([
                timestamp,
                datapoint,
            ])

    return ticktime_series


def get_mean_ticktime_from_series(ticktime_series):
    if len(ticktime_series) > 0:
        ticktimes = [Decimal(t[1]) for t in ticktime_series]

        return sum(ticktimes) / len(ticktimes)
    else:
        return 0


def get_tick_block_time_series_for_exchange(db, exchange_name, start_time, end_time):
    block_datum_type = exchange_name.upper() + '_TICK_BLOCK_TIME%'

    block_times = db.query(Datum)\
        .filter(Datum.time_created >= start_time)\
        .filter(Datum.time_created < end_time)\
        .filter(Datum.datum_type.like(block_datum_type))\
        .all()

    series = sort_blocks_into_series(block_times)

    return series


def get_block_name_from_datum_type(datum_type):
    return datum_type[datum_type.index('BLOCK_TIME_') + 11:]


def sort_blocks_into_series(block_times):
    block_series = defaultdict(lambda: [])

    for block_time in block_times:
        block_name = get_block_name_from_datum_type(block_time.datum_type)
        timestamp = Delorean(block_time.time_created, "UTC").epoch * 1000
        datapoint = str(block_time.numeric_value)

        block_series[block_name].append([
            timestamp,
            datapoint,
        ])

    return block_series



# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict
import decimal
import hashlib
import hmac
import json
import time

from cdecimal import *

from base import *
from exceptions import *
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money

from gryphon.lib.logger import get_logger
logger = get_logger(__name__)


class PoloniexETHBTCExchange(Exchange):
    def __init__(self, session=None, use_cached_orderbook=False):
        super(PoloniexETHBTCExchange, self).__init__(session)
        self.name = u'POLONIEX_ETH_BTC'
        self.friendly_name = u'Poloniex ETH-BTC'
        self.currency = 'BTC'
        self.volume_currency = 'ETH'




from datetime import timedelta

from gryphon.lib.models.trade import Trade
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money

import sqlalchemy


"""
    Gives the portion of time that all exchanges in the given list were up
        in the given period.

    That is, the number of ten minute periods in the window in which every 
        one of these exchanges place an order.
"""
def global_uptime_for_exchanges_in_period(db, exchange_names, start_time, end_time):
    all_buckets = all_buckets_in_period(start_time, end_time)

    exchange_up_buckets = {}

    for exchange_name in exchange_names:
        exchange_up_buckets[exchange_name] = exchange_up_buckets_in_period(
            db,
            exchange_name,
            start_time,
            end_time,
        )  

    down_periods = 0.0

    for bucket in all_buckets:
        for exchange_name in exchange_names:
            if bucket not in exchange_up_buckets[exchange_name]:
                down_periods = down_periods + 1
                break

    downtime = down_periods / len(all_buckets)

    uptime = 1 - downtime

    return uptime


"""
    Gives a list of all the ten-minute periods in the given time range in
        which the given exchange placed orders.

    Similar to exchange_uptime_in_period except this gives you the actual
        list of times.
"""
def exchange_up_buckets_in_period(db, exchange_name, start_time, end_time):
    # this query gets a list of timestamps indicating the start of all
    # ten-minute periods in the given time window in which the exchange
    # bot placed at least one order
    up_buckets = db.query('time_period')\
        .from_statement("select from_unixtime(bucket) as time_period, count(*) as num_orders from (select unix_timestamp(`order`.time_created) - (unix_timestamp(`order`.time_created) % 600) as bucket from `order` where `order`.exchange_name = :exchange_name and `order`.time_created >= :start_time and `order`.time_created < :end_time) as buckets group by bucket")\
        .params(
            exchange_name=exchange_name,
            start_time=start_time,
            end_time=end_time,
        )\
        .all()

    up_buckets = [u[0] for u in up_buckets]

    return up_buckets


"""
    Gives the portion of time in the given period that the given exchange
        was up.

    An exchange is said to be "up" if it places at least one order in a 
        ten minute period.
"""
def exchange_uptime_in_period(db, exchange_name, start_time, end_time):
    num_periods = (end_time - start_time).total_seconds() / 600

    uptime = db.query('uptime')\
        .from_statement("select count(*)/:num_periods as uptime from (select from_unixtime(bucket) as time_period, count(*) as num_orders from (select unix_timestamp(`order`.time_created) - (unix_timestamp(`order`.time_created) % 600) as bucket from `order` where `order`.exchange_name = :exchange_name and `order`.time_created >= :start_time and `order`.time_created < :end_time) as buckets group by bucket) as up_buckets")\
        .params(
            num_periods=num_periods,
            exchange_name=exchange_name,
            start_time=start_time,
            end_time=end_time,
        )\
        .first()

    return uptime[0]


"""
    Helper function gives us a list of all the ten-minute periods between
        start_time and end_time.
"""
def all_buckets_in_period(start_time, end_time):
    num_periods = int((end_time - start_time).total_seconds() / 600)

    start_bucket = start_time - timedelta(minutes=start_time.minute % 10)

    all_buckets = [start_bucket + timedelta(minutes=10*i) for i in range(0,num_periods)]

    return all_buckets





import numpy as np
import time
import math

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)

def GAPO(d,h,l,tf):
    gapo = []
    x = tf

    while x < len(d):
        consHigh = h[x-tf:x]
        consLow = l[x-tf:x]

        HighestHigh = max(consHigh)
        LowestLow = min(consLow)

        gapos = ( (math.log(HighestHigh - LowestLow)) /
                  math.log(tf))

        print gapos
        gapo.append(gapos)
        x+=1
    return d[tf:],gapo

GAPO(date,highp,lowp,14)



from twisted.python import log

from gryphon.data_service.pollers.volume.volume_poller import VolumePoller


class CoinbaseVolume(VolumePoller):
    def __init__(self):
        self.exchange_name = u'COINBASE_BTC_USD'
        self.url = 'https://api.pro.coinbase.com/products/BTC-USD/stats'
        self.poll_time = 2

    def parse_volume_json(self, volume_dict):
        try:
            return volume_dict['volume']
        except Exception as e:
            log.msg('Error getting Coinbase Volume %s' % e)



# -*- coding: utf-8 -*-
from sqlalchemy import Column, Unicode, UnicodeText

from gryphon.lib.models.base import Base

metadata = Base.metadata

class Flag(Base):
    __tablename__ = 'flag'

    key = Column(Unicode(64), primary_key=True)
    value = Column(UnicodeText(length=2**31))

    def __init__(self, key, value):
        self.key = key
        self.value = value
        



# -*- coding: utf-8 -*-
from datetime import datetime
import json
import os
import uuid

from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, BigInteger, Numeric

from gryphon.lib.models.base import Base
from gryphon.lib.session import commit_mysql_session
from gryphon.lib.singleton import Singleton

metadata = Base.metadata


class Event(Base):
    __tablename__ = 'event'
   
    unique_id = Column(Unicode(64), nullable=False)
    event_id = Column(Integer, primary_key=True)
    time_created = Column(DateTime, nullable=False)
    exchange_name = Column(Unicode(256), nullable=False)
    event_type = Column(Unicode(256), nullable=False)
    data = Column(UnicodeText(length=2**31))
 
    def __init__(self, event_type, exchange_name, data):
        self.time_created = datetime.utcnow()
        self.event_type = event_type
        self.exchange_name = exchange_name
        self.unique_id = u'evt_%s' % unicode(uuid.uuid4().hex)
        self.data = json.dumps(data)
        
    def __unicode__(self):
        return unicode(repr(self))
        
    def __repr__(self):
        return json.dumps({
            'event_type':self.event_type,
            'exchange':self.exchange_name,
            'data': json.loads(self.data),
            'time_created': unicode(self.time_created),
        }, ensure_ascii=False)


class EventRecorder(object):
    __metaclass__ = Singleton

    def create(self, db=None, logger=None):
        self.db = db
        self.external_logger = logger
        
    def record(self, event_type, exchange_name='', data={}):
        event = Event(event_type, exchange_name, data)
        
        if not hasattr(self, 'db') and not hasattr(self, 'external_logger'):
            # we didn't call create. we aren't recording events
            return
        
        if self.db:
            self.db.add(event)
            commit_mysql_session(self.db)
        elif self.external_logger:
            self.external_logger.info('[EVENT] %s : %s : %s -- %s' % (event.exchange_name, event.event_type, str(event.time_created), json.dumps(event.data)))
        else:
            # we aren't recording events.
            pass
            

                



from delorean import Delorean
from twisted.internet import defer
from twisted.internet.task import LoopingCall
from twisted.python import log
from twisted.python.filepath import FilePath

from gryphon.data_service.auditors.auditor import Auditor
import gryphon.data_service.util as util


class HeartbeatAuditor(Auditor):
    @defer.inlineCallbacks
    def start(self):
        self.touch_path = 'monit/heartbeat/twisted.txt'
        self.redis = yield util.setup_redis()

        self.heartbeat_threshold = 120
        self.heartbeats = [
            'bitfinex_trades_heartbeat',
            'bitstamp_btc_usd_trades_heartbeat',
            'coinbase_btc_usd_trades_heartbeat',
            'itbit_trades_heartbeat',
            'kraken_trades_heartbeat',
            'kraken_usd_trades_heartbeat',
            'kraken_cad_trades_heartbeat',
            'okcoin_trades_heartbeat',
            #'coinbase_cad_trades_heartbeat',
            'gemini_trades_heartbeat',
            'quadriga_trades_heartbeat',

            'okcoin_orderbook_heartbeat',
            'bitstamp_btc_usd_orderbook_heartbeat',
            'kraken_orderbook_heartbeat',
            'kraken_usd_orderbook_heartbeat',
            'kraken_cad_orderbook_heartbeat',
            'itbit_orderbook_heartbeat',
            'coinbase_btc_usd_orderbook_heartbeat',
            #'coinbase_cad_orderbook_heartbeat',
            'bitfinex_orderbook_heartbeat',
            'quadriga_orderbook_heartbeat',
            'gemini_orderbook_heartbeat',

            'gemini_eth_btc_orderbook_heartbeat',
            'bitstamp_eth_btc_orderbook_heartbeat',
            'poloniex_eth_btc_orderbook_heartbeat',

            'gemini_eth_usd_orderbook_heartbeat',
            'bitstamp_eth_usd_orderbook_heartbeat',

            'bitstamp_eth_eur_orderbook_heartbeat',
            'bitstamp_bch_eur_orderbook_heartbeat',
            'bitstamp_bch_usd_orderbook_heartbeat',
            'bitstamp_bch_btc_orderbook_heartbeat',
            'bitstamp_btc_eur_orderbook_heartbeat',

            'open_exchange_rate_poller_heartbeat',
        ]

        self.looping_call = LoopingCall(self.audit)
        self.looping_call.start(30)

    @defer.inlineCallbacks
    def audit(self):
        successful_heartbeat_check = True
        now = Delorean().epoch

        for heartbeat_key in self.heartbeats:
            last_heartbeat = yield self.redis.get(heartbeat_key)
            last_heartbeat = float(last_heartbeat) if last_heartbeat else 0
            heartbeat_age = now - last_heartbeat

            if heartbeat_age > self.heartbeat_threshold:
                log.msg(
                    '[Heartbeat Auditor] Failure %s: %ds is older than %ds' % (
                    heartbeat_key,
                    heartbeat_age,
                    self.heartbeat_threshold,
                ))

                successful_heartbeat_check = False

        if successful_heartbeat_check:
            log.msg('[Heartbeat Auditor] Passed')
            FilePath(self.touch_path).touch()
        else:
            log.msg('[Heartbeat Auditor] Failed')



"""
Just a small module to centralize our handling of .env files in gryphon. Currently all
our executables look for a file named '.env' in the current working directory. This
could be made more sophisticated in the future, e.g., looking for ~/.gryphon/.env first,
and allowing a cwd .env file to override. Lots of options.
"""

import os
from os.path import join

from dotenv import load_dotenv


def load_environment_variables():
    """
    Presently, this function looks for a .env file in the current directory and loads
    the variables present therin into os.environ.
    """
    dotenv_path = join(os.getcwd(), '.env')
    load_dotenv(dotenv_path)



# -*- coding: utf-8 -*-
import copy
import json
import uuid

from delorean import parse
from sqlalchemy import Column, Integer, Unicode, Numeric, PickleType
from sqlalchemy import or_, and_
from sqlalchemy import func
from sqlalchemy.ext.mutable import Mutable
from sqlalchemy.orm import relationship, reconstructor, Session
from sqlalchemy.types import TypeDecorator, UnicodeText

from gryphon.lib import gryphon_json_serialize
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.logger import get_logger
from gryphon.lib.models.base import Base
from gryphon.lib.models.order import Order
from gryphon.lib.models.trade import Trade
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money

logger = get_logger(__name__)

metadata = Base.metadata


class JSONEncodedMoneyDict(TypeDecorator):
    """
    This class handles storage and retrieval of certain data in the database,
    specifically dictionaries wherein all keys are Money objects. This is used in
    conjuction with MutableDict below to store balance information for exchange accounts
    and a few other things.
    """

    impl = UnicodeText(length=2**31)

    def load_json_money_dict(self, value):
        d = json.loads(value)

        for key, value in d.items():
            d[key] = Money.loads(value)

        return d

    def process_bind_param(self, value, dialect):
        if value is not None:
            value = json.dumps(value)
        return value

    def process_result_value(self, value, dialect):
        if value is not None:
            value = self.load_json_money_dict(value)
        return value


class MutableDict(Mutable, dict):
    """
    Used in conjunction with JSONMoneyDict as above, the MutableDict constructor is
    called when we load the mutable columns. This calls coerce(cls, key, value), where
        cls ~ is the Class of the column (Balance, Position, Target0,
        key ~ is the name of the column
        value ~ is the result that's been parsed by JSONEncodedMoneyDict.

    This is all based on sqlalchemy documentation here:
        https://docs.sqlalchemy.org/en/latest/orm/extensions/mutable.html

    The examples for MutableDict have a strange control flow to them that I don't
    understand, but the result that we want is for these to be available as their proper
    Balance/Position/Target classes when we load them, and this can be achieved just by
    doing return cls(value) in the coerce method.
    """
    @classmethod
    def coerce(cls, key, value):
        if isinstance(value, dict) and cls in [Balance, Position, Target]:
            return cls(value)
        else:
            # This will error.
            return Mutable.coerce(key, value)

    def __delitem(self, key):
        dict.__delitem__(self, key)
        self.changed()

    def __setitem__(self, key, value):
        dict.__setitem__(self, key, value)
        self.changed()

    def __getstate__(self):
        return dict(self)

    def __setstate__(self, state):
        self.update(self)


class Balance(MutableDict):
    def __missing__(self, key):
        value = self[key] = Money(0, key)
        return value

    def __add__(self, other):
        # Make a deep copy of self. copy.deepcopy() is 4x slower than this
        # because we know the exact structure and don't have to watch for recursions
        result = self.__class__()
        for currency, m in self.iteritems():
            result[currency] = Money(m.amount, m.currency)

        if isinstance(other, Balance):
            all_currencies = list(set(self.keys() + other.keys()))
            for c in all_currencies:
                result[c] += other[c]
        elif isinstance(other, Money):
            result[other.currency] += other
        else:
            raise ValueError('Can only add Money and Balance objects')
        return result

    def __neg__(self):
        result = self.__class__()
        for currency, balance in self.iteritems():
            result[currency] = -balance
        return result

    def __sub__(self, other):
        return self + (-other)

    def fiat(self):
        currencies = self.keys()

        non_btc_currencies = [c for c in currencies if c != 'BTC']
        if len(non_btc_currencies) > 1:
            raise ValueError('Balance has more than one fiat currency')
        elif len(non_btc_currencies) < 1:
            return None

        fiat_currency = non_btc_currencies[0]
        return self[fiat_currency]

    def total_usd_value(self, date=None):
        total_usd_value = Money(0, 'USD')
        for currency, balance in self.iteritems():
            total_usd_value += balance.to('USD', date=date)
        return total_usd_value


class Position(Balance):
    pass


class Target(Balance):
    pass


class Exchange(Base):
    __tablename__ = 'exchange'

    exchange_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    name = Column(Unicode(64), nullable=False)

    position = Column(Position.as_mutable(JSONEncodedMoneyDict))
    target = Column(Target.as_mutable(JSONEncodedMoneyDict))
    balance = Column(Balance.as_mutable(JSONEncodedMoneyDict))

    _multi_position_cache = Column('multi_position_cache', Numeric(precision=24, scale=14))

    # lazy=dynamic makes this behave like a query
    transactions = relationship('Transaction', cascade='all,delete-orphan', backref='exchange', lazy='dynamic')

    def __init__(self, name):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.name = name
        self.position = Position()
        self.target = Target()
        self.balance = Balance()
        self.multi_position_cache = Money(0, 'BTC')

    @reconstructor
    def init_on_load(self):
        if not self.position:
            self.position = Position()
        if not self.target:
            self.target = Target()
        if not self.balance:
            self.balance = Balance()

    def __unicode__(self):
        return u'[EXCHANGE: %s] Position: %s Target: %s Balance: %s' % (
           self.name, self.position, self.target, self.balance)

    def __repr__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps({
            'name': self.name,
            'exchange_id': self.exchange_id,
            'unique_id': self.unique_id,
            'position': self.position,
            'target': self.target,
            'balance': self.balance,
        }, ensure_ascii=False)

    @property
    def multi_position_cache(self):
        return Money(self._multi_position_cache, 'BTC')

    @multi_position_cache.setter
    def multi_position_cache(self, value):
        if value.currency != 'BTC':
            raise ValueError('multi_position_cache must be a BTC Money object')
        self._multi_position_cache = value.amount

    def record_fiat_withdrawal(self, target_exchange_data, withdrawal_amount, deposit_amount=None, transaction_details={}):
        if deposit_amount:
            logger.info('deposit amount: %s' % deposit_amount)
            if deposit_amount.currency != withdrawal_amount.currency:
                exchange_rate = str(withdrawal_amount.amount / deposit_amount.amount)
                transaction_details.update({'exchange_rate': exchange_rate})

        withdrawal_fee = None
        source_exchange = make_exchange_from_key(self.name)
        # We don't have exchange objects for bank accounts (BMO_USD, BMO_CAD)
        if source_exchange:
            withdrawal_fee = source_exchange.fiat_withdrawal_fee(withdrawal_amount)
            # assumes that fees are taken out of the withdrawal amount instead of the balance
            withdrawal_amount -= withdrawal_fee

        withdrawal = Transaction(Transaction.WITHDRAWL, Transaction.IN_TRANSIT, withdrawal_amount, self, transaction_details, fee=withdrawal_fee)
        withdrawal.complete()

        if deposit_amount is None:
            deposit_amount = withdrawal_amount

        deposit_fee = None
        target_exchange = make_exchange_from_key(target_exchange_data.name)
        if target_exchange:
            deposit_fee = target_exchange.fiat_deposit_fee(deposit_amount)

        deposit = Transaction(Transaction.DEPOSIT, Transaction.IN_TRANSIT, deposit_amount, target_exchange_data, transaction_details, fee=deposit_fee)

        return (deposit, withdrawal)

    def record_withdrawal(self, target_exchange_data, amount, deposit_address='', transaction_hash='', exchange_withdrawal_id=''):
        details = {}
        if deposit_address:
            details['deposit_address'] = deposit_address
        if transaction_hash:
            details['transaction_hash'] = transaction_hash
        if exchange_withdrawal_id:
            details['exchange_withdrawal_id'] = exchange_withdrawal_id

        exchange = make_exchange_from_key(self.name)
        withdrawal_fee = exchange.withdrawal_fee
        deposit = Transaction(Transaction.DEPOSIT, Transaction.IN_TRANSIT, amount, target_exchange_data, details)
        withdrawl = Transaction(Transaction.WITHDRAWL, Transaction.IN_TRANSIT, amount, self, details, fee=withdrawal_fee)
        withdrawl.complete()
        return (deposit, withdrawl)

    def pending_deposits(self, currency=None):
        query = self.transactions.filter(Transaction.transaction_type == Transaction.DEPOSIT).filter(Transaction.transaction_status == Transaction.IN_TRANSIT)
        if currency:
            query = query.filter(Transaction._amount_currency == currency)
        return query.all()

    def combinations_of_all_lengths(self, items):
        result = []
        from itertools import combinations
        for comb_length in range(1, len(items) + 1):
            combs = combinations(items, comb_length)
            result += combs
        return result

    def deposit_landed(self, amount_changed, tolerance=0):
        expected_deposits = self.pending_deposits(amount_changed.currency)
        possible_combinations = self.combinations_of_all_lengths(expected_deposits)
        for c in possible_combinations:
            total_amount = 0
            for deposit in c:
                total_amount += deposit.amount
                if deposit.fee:
                    total_amount -= deposit.fee
            # we found a combination of deposits which match the amount changed
            # this should have a tolerance
            if abs(total_amount - amount_changed) <= tolerance:
                for deposit in c:
                    deposit.complete()

                deposit_ids = [int(d.transaction_id) for d in c]
                logger.info('Deposits %s for %s match up to the %s change in balance' % (deposit_ids, total_amount, amount_changed))
                return True
        return False

    @property
    def full_balance(self):
        in_transit_transactions = self.pending_deposits()

        in_transit_balance = copy.deepcopy(self.balance)
        for tr in in_transit_transactions:
            in_transit_balance[tr.amount.currency] += tr.amount
        return in_transit_balance

    @property
    def trades(self):
        session = Session.object_session(self)

        return session.query(Trade)\
            .join(Order)\
            .filter(Order._exchange_name.in_(self.all_pair_names))

    def ledger(self, start_time, end_time, currency=None):
        """
        Load the most recent <limit> entries in the ledger (Trades or Transactions).

        Can filter to only entries which affect a specific <currency>.
        """
        from gryphon.lib import assets

        if start_time and start_time < assets.EARLIEST_CORRECT_LEDGER_DATE:
            raise ValueError(
                'start_time must be later than %s' % (
                    assets.EARLIEST_CORRECT_LEDGER_DATE,
                ))

        if end_time and end_time < assets.EARLIEST_CORRECT_LEDGER_DATE:
            raise ValueError(
                'end_time must be later than %s' % assets.EARLIEST_CORRECT_LEDGER_DATE,
            )

        trades_query = self.trades\
            .order_by(Trade.time_created.desc())

        if currency:
            trades_query = trades_query.filter(
                or_(
                    Trade._price_currency == currency,
                    Trade._volume_currency == currency,
                    Trade._fee_currency == currency,
                ),
            )

        if start_time:
            trades_query = trades_query.filter(Trade.time_created > start_time)

        if end_time:
            trades_query = trades_query.filter(Trade.time_created <= end_time)

        trades = trades_query.all()

        transactions_query = self.transactions\
            .filter(Transaction.transaction_status == Transaction.COMPLETED)\
            .order_by(Transaction.time_completed.desc())

        if currency:
            transactions_query = transactions_query.filter(
                or_(
                    Transaction._amount_currency == currency,
                    Transaction._fee_currency == currency,
                ),
            )

        if start_time:
            transactions_query = transactions_query.filter(or_(
                and_(
                    Transaction.time_completed != None,
                    Transaction.time_completed > start_time,
                ),
                and_(
                    Transaction.time_completed == None,
                    Transaction.time_created > start_time,
                ),
            ))

        if end_time:
            # We include transactions in the ledger based on when they were completed,
            # not created. This is an issue because we only started recording
            # time_completed on 2015-10-08. So for these pre-time_completed transactions
            # we use their time_created instead.
            transactions_query = transactions_query.filter(or_(
                and_(
                    Transaction.time_completed != None,
                    Transaction.time_completed <= end_time,
                ),
                and_(
                    Transaction.time_completed == None,
                    Transaction.time_created <= end_time,
                ),
            ))

        transactions = transactions_query.all()

        def sort_key(t):
            if isinstance(t, Trade):
                return t.time_created
            elif isinstance(t, Transaction):
                if t.time_completed:
                    return t.time_completed
                else:
                    return t.time_created

        entries = sorted(trades + transactions, key=sort_key, reverse=True)

        return entries

    @property
    def all_pair_names(self):
        # Just a hack until we migrate the db.

        if self.name == 'GEMINI':
            return [self.name, 'GEMINI_ETH_USD']
        else:
            return [self.name]

    def ledger_balance(self, start_time=None, end_time=None, include_pending=False):
        from gryphon.lib import assets
        session = Session.object_session(self)

        return assets.ledger_balance(
            session,
            start_time=start_time,
            end_time=end_time,
            include_pending=include_pending,
            exchange_names=self.all_pair_names,
        )



from gryphon.execution.strategies.builtin.fundamental_value import v1
from gryphon.execution.strategies.builtin.fundamental_value import v2
from gryphon.execution.strategies.builtin.fundamental_value import v3
from gryphon.execution.strategies.builtin.fundamental_value import native



# -*- coding: utf-8 -*-
import os
from gryphon.lib.models.emeraldhavoc.base import EmeraldHavocBase
import json
import uuid
from decimal import *
from collections import defaultdict

from gryphon.lib.exchange.consts import Consts
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib import gryphon_json_serialize
from datetime import datetime, date, timedelta
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base

metadata = EmeraldHavocBase.metadata

class Orderbook(EmeraldHavocBase):
    __tablename__ = 'orderbook'
    
    orderbook_id = Column(Integer, primary_key=True)
    unique_id = Column(Unicode(64), nullable=False)
    exchange = Column(Unicode(64), nullable=False)
    timestamp = Column(DateTime, nullable=True)
    time_created = Column(DateTime, nullable=False)
    _bids = Column('bids', UnicodeText(length=2**31))
    _asks = Column('asks', UnicodeText(length=2**31))
    
    def __init__(self, exchange, orderbook, timestamp=None):
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()
        self.timestamp = timestamp
        self.bids = orderbook['bids']
        self.asks = orderbook['asks']
        self.exchange = exchange
        
    def __unicode__(self):
        bids = self.bids[:5]
        asks = self.asks[:5]
        output = u"\n%s\nBids\n" % self.exchange
        output += u"{0:15} | {1:15}\n".format("Price", "Amount")
        for bid in bids:
            output += u"{0:15} | {1:15}\n".format(bid[0], bid[1])

        output += u"Asks\n"
        output += u"{0:15} | {1:15}\n".format("Price", "Amount")

        for ask in asks:
            output += u"{0:15} | {1:15}\n".format(ask[0], ask[1])
        
        return output
        
    def __repr__(self):
        return self.to_json()
    
    def to_json(self):
        return json.dumps({
            'trade_id':self.trade_id,
            'time_created':unicode(self.time_created),
            'unique_id':self.unique_id,
            'exchange':self.exchange,
            'bids':self.bids,
            'asks':self.asks
        }, ensure_ascii=False)
    
    @property
    def bids(self):
        return json.loads(self._bids)

    @bids.setter
    def bids(self, value):
        self._bids = json.dumps(value, ensure_ascii=False)

    @property
    def asks(self):
        return json.loads(self._asks)

    @asks.setter
    def asks(self, value):
        self._asks = json.dumps(value, ensure_ascii=False)





import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)



def ExpMovingAverage(values, window):
    weights = np.exp(np.linspace(-1., 0., window))
    weights /= weights.sum()
    a =  np.convolve(values, weights, mode='full')[:len(values)]
    a[:window] = a[window]
    return a

def EFI(d,c,v,tf):
    efi = []

    x = 1
    while x < len(d):
        forceIndex = (c[x] - c[x-1]) * v[x]
        print forceIndex
        efi.append(forceIndex)
        x+=1
    efitf = ExpMovingAverage(efi,tf)

    return d[1:], efitf

efix,efiy = EFI(date,closep,volume,14)



import ConfigParser
from delorean import Delorean
import pickle
import random
import termcolor as tc

from cdecimal import Decimal, InvalidOperation, ROUND_UP, ROUND_DOWN

from gryphon.execution.brain import Brain
from gryphon.lib.configurable_object import ConfigurableObject
from gryphon.lib.exchange.consts import Consts
import gryphon.lib.gryphonfury.positions as positions
from gryphon.lib.logger import get_logger
from gryphon.lib.models.datum import DatumRecorder
from gryphon.lib.models.flag import Flag
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money
from gryphon.lib.session import commit_mysql_session


logger = get_logger(__name__)


MIN_VOLUME_OBFUSCATION = 0.00
MAX_VOLUME_OBFUSCATION = 0.10


class Strategy(ConfigurableObject):
    def __init__(self, db, harness=None, strategy_configuration=None):
        self.db = db
        self.harness = harness

        self.brain = Brain()
        self._position = None
        self.order_class = Order

        # Configurable properties with defaults only below this line.

        self.volume_currency = 'BTC'

        # How long the strategy pauses between ticks.
        self.tick_sleep = 5

        # Exchanges that the strategy declares it's interested in up-front. It's
        # convenient to have these at startup sometimes.
        self.target_exchanges = []  

        if strategy_configuration:
            self.configure(strategy_configuration)

    def configure(self, strategy_configuration):
        """
        Initialize the strategy's configurable properties based on the configuration
        given to us by the harness, which has already been synthesized from the
        command line and .conf file.
        """
        self.init_configurable('tick_sleep', strategy_configuration)

    def set_up(self):
        """
        In this function do any setup that needs to be done on startup, before we enter
        the tick-loop, that may not be appropriate in the constructor.
        """
        #self.load_brain()

    def pre_tick(self):
        self._max_position = None

        # This is necessary because _position functions as an in-tick cache, so this
        # line clears that cache from last tick. Could also be done in post_tick.
        self._position = None

    def tick(self, order_book, eaten_orders):
        raise NotImplementedError

    def post_tick(self, tick_count):
        self.save_brain()

    def is_complete(self):
        """
        Strategies can signal to their runners that they are "finished". This isn't
        usually relevant for continuous-trading strategies like a market maker, but is
        very useful for execution strategies, for instance.
        """
        return False

    def load_brain(self):
        if self.db:
            self.harness.log('Loading Brain')

            key = '%s_brain' % self.name
            flag = self.db.query(Flag).filter_by(key=key).first()

            if flag:
                self.brain = pickle.loads(str(flag.value))

    def save_brain(self):
        if self.db:
            self.harness.log('Saving Brain', log_level='debug')

            key = '%s_brain' % self.name
            flag = self.db.query(Flag).filter_by(key=key).first()
            data = pickle.dumps(self.brain)

            if flag:
                flag.value = data
            else:
                flag = Flag(key, data)

            self.db.add(flag)

    @property
    def actor(self):
        """
        The strategy's 'actor' is how orders and trades are associated with the
        strategy in the database. As a consequence, if two strategies have the same
        actor, they have the same trade history and position.
        """
        return self.__class__.__name__.upper()

    @property
    def name(self):
        """
        The strategy's 'name' is a less formal identifier for a strategy than it's
        actor, and has nothing to do with the operation of strategies in the framework.
        Use it for things that aren't mission-critical, like to make log messages nice.
        """
        return self.__class__.__name__
   
    @property 
    def position(self):
        if self._position is not None:
            return self._position

        # Currently we just default to assuming the class's actor is it's classname,
        # but we'll improve this shortly.

        self._position = positions.fast_position(
            self.db,
            volume_currency=self.volume_currency,
            actor=self.actor,
        )

        return self._position

    def max_position(self):
        # self._max_position gets reset on every tick, so we are just memoizing it for
        # 1 tick.

        if self._max_position == None:
            # We could run some analysis here that would change our max_position for
            # 1 tick.
            self._max_position = self.config.get('max_position')

        return self._max_position

    def are_different_enough(self, old, new, diff=Money('0.01', 'CAD')):
        assert(old.currency == new.currency)

        diff = diff.to(old.currency)

        return diff < abs(old - new)




import termcolor as tc
import time

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.strategies.harness import Harness
from gryphon.execution.lib.exchange_color import exchange_color
from gryphon.lib import session
from gryphon.lib.exchange.exchange_factory import *
from gryphon.lib.logger import get_logger

logger = get_logger(__name__)


def wind_down(exchange_name, strategy_params, execute=False):
    db = session.get_a_trading_db_mysql_session()

    try:
        strategy = Strategy(
            make_exchange_from_key(exchange_name),
            db,
            debug=False,
            backtest=False,
            execute=execute,
            params=strategy_params,
        )

        harness = Harness(strategy, db)
        harness.wind_down()
    finally:
        session.commit_mysql_session(db)
        db.remove()



"""
Exchange documentation: https://api.itbit.com/docs
"""

# -*- coding: utf-8 -*-
import base64
from collections import OrderedDict, defaultdict
import hashlib
import hmac
import json
import time
import urllib

import cdecimal
from cdecimal import Decimal

from gryphon.lib.exchange import exceptions
from gryphon.lib.exchange import order_types
from gryphon.lib.exchange.exchange_api_wrapper import ExchangeAPIWrapper
from gryphon.lib.logger import get_logger
from gryphon.lib.models.exchange import Balance
from gryphon.lib.money import Money
from gryphon.lib.time_parsing import parse

logger = get_logger(__name__)


class ItbitBTCUSDExchange(ExchangeAPIWrapper):
    def __init__(self, session=None, configuration=None):
        super(ItbitBTCUSDExchange, self).__init__(session)

        self.name = u'ITBIT_BTC_USD'
        self.friendly_name = u'Itbit BTC-USD'
        self.base_url = 'https://api.itbit.com/v1'
        self.currency = 'USD'
        self.bid_string = 'buy'
        self.ask_string = 'sell'
        self.nonce = 1

        # Configurables with defaults.
        self.market_order_fee = Decimal('0.002')
        self.limit_order_fee = Decimal('0')
        self.fee = self.market_order_fee
        self.fiat_balance_tolerance = Money('0.0001', 'USD')
        self.volume_balance_tolerance = Money('0.00000001', 'BTC')
        self.max_tick_speed = 1
        self.min_order_size = Money('0', 'BTC')
        self.use_cached_orderbook = False

        if configuration:
            self.configure(configuration)

    @property
    def wallet_id(self):
        try:
            self._wallet_id
        except AttributeError:
            self._wallet_id = self._load_env('ITBIT_BTC_USD_WALLET_ID')

        return self._wallet_id

    def req(self, req_method, url, **kwargs):
        # Our auth_request method expects the params in the url.
        assert '?' not in url

        if 'params' in kwargs:
            if kwargs['params']: # Check that it's not empty.
                url += '?' + urllib.urlencode(kwargs['params'])

            del kwargs['params']

        req = super(ItbitBTCUSDExchange, self).req(req_method, url, **kwargs)

        return req

    def resp(self, req):
        response = super(ItbitBTCUSDExchange, self).resp(req)

        if 'error' in response and response['error']:
            raise exceptions.ExchangeAPIErrorException(self, response['error'])

        if 'code' in response:
            errors_string = str(response['description'])
            error_code = int(response['code'])

            if error_code == 81001:
                raise exceptions.InsufficientFundsError()
            elif error_code == 10002:
                raise exceptions.NonceError()
            elif error_code == 81002:
                raise exceptions.CancelOrderNotFoundError()
            else:
                raise exceptions.ExchangeAPIErrorException(
                    self,
                    'Code %s: %s' % (
                    error_code, errors_string,
                ))

        return response

    def all_trades(self, page=1):
        req = self.all_trades_req(page)
        return self.all_trades_resp(req)

    def all_trades_req(self, page=1):
        params = {}

        if page:
            params['page'] = page

        return self.req(
            'get',
            '/wallets/%s/trades' % self.wallet_id,
            params=params,
        )

    def all_trades_resp(self, req):
        response = self.resp(req)
        return response['tradingHistory']

    def trades_for_orders(self, order_ids):
        req = self.trades_for_orders_req()
        return self.trades_for_orders_resp(req, order_ids)

    def trades_for_orders_req(self):
        return self.all_trades_req()

    def trades_for_orders_resp(self, req, order_ids):
        order_ids = [str(o) for o in order_ids]
        trades = self.all_trades_resp(req)

        matching_trades = defaultdict(list)

        for trade in trades:
            oid = str(trade['orderId'])

            if oid in order_ids:
                matching_trades[oid].append(trade)

        return matching_trades

    def all_orders(self, status=None, page=1):
        req = self.all_orders_req(status, page)
        return self.all_orders_resp(req)

    def all_orders_req(self, status=None, page=1):
        params = {}

        if status:
            params['status'] = status
        if page:
            params['page'] = page

        return self.req(
            'get',
            '/wallets/%s/orders' % self.wallet_id,
            params=params,
        )

    def all_orders_resp(self, req):
        raw_orders = self.resp(req)
        orders = []

        for raw_order in raw_orders:
            mode = self._order_mode_to_const(raw_order['side'])
            volume = Money(raw_order['amount'], 'BTC')
            volume_filled = Money(raw_order['amountFilled'], 'BTC')
            volume_remaining = volume - volume_filled

            order = {
                'mode': mode,
                'id': str(raw_order['id']),
                'price': Money(raw_order['price'], 'USD'),
                'volume': volume,
                'volume_remaining': volume_remaining,
                'status': raw_order['status']
            }

            orders.append(order)

        return orders

    # Common Exchange Methods

    def auth_request(self, req_method, url, request_args):
        """
        This modifies request_args.
        """
        try:
            self.api_key
            self.secret
        except AttributeError:
            self.api_key = self._load_env('ITBIT_BTC_USD_API_KEY')
            self.secret = self._load_env('ITBIT_BTC_USD_API_SECRET').encode('utf-8')

        timestamp = int(round(time.time() * 1000))
        nonce = self.nonce

        body = ''

        if 'data' in request_args:
            body = json.dumps(request_args['data'])

        request_args['data'] = body

        message = self._auth_create_message(req_method, url, body, nonce, timestamp)

        sig = self._auth_sign_message(message, nonce, url, self.secret)

        if 'headers' not in request_args:
            request_args['headers'] = {}

        headers = request_args['headers']

        headers['Authorization'] = self.api_key + ':' + sig
        headers['X-Auth-Timestamp'] = str(timestamp)
        headers['X-Auth-Nonce'] = str(nonce)
        headers['Content-Type'] = 'application/json'

    def _auth_create_message(self, verb, url, body, nonce, timestamp):
        return json.dumps(
            [verb.upper(), url, body, str(nonce), str(timestamp)],
            separators=(',', ':'),
        )

    def _auth_sign_message(self, message, nonce, url, api_secret):
        sha256_hash = hashlib.sha256()
        nonced_message = str(nonce) + message
        sha256_hash.update(nonced_message)
        hash_digest = sha256_hash.digest()

        msg_to_hmac = url.encode('utf8') + hash_digest
        hmac_digest = hmac.new(api_secret, msg_to_hmac, hashlib.sha512).digest()

        sig = base64.b64encode(hmac_digest)

        return sig

    def get_balance_req(self):
        try:
            self.user_id
        except AttributeError:
            self.user_id = self._load_env('ITBIT_BTC_USD_USER_ID')

        return self.req('get', '/wallets/%s' % self.wallet_id)

    def get_balance_resp(self, req):
        response = self.resp(req)
        raw_balances = response['balances']

        btc_available = None
        usd_available = None

        for raw_balance in raw_balances:
            if raw_balance['currency'] == 'XBT':
                btc_available = Money(raw_balance['availableBalance'], 'BTC')
            elif raw_balance['currency'] == 'USD':
                usd_available = Money(raw_balance['availableBalance'], 'USD')

        if btc_available is None or usd_available is None:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'missing expected balances',
            )

        balance = Balance()
        balance['BTC'] = btc_available
        balance['USD'] = usd_available

        return balance

    def get_ticker_req(self, verify=True):
        return self.req(
            'get',
            '/markets/XBTUSD/ticker',
            no_auth=True,
            verify=verify,
        )

    def get_ticker_resp(self, req):
        response = self.resp(req)

        return {
            'high': Money(response['high24h'], 'USD'),
            'low': Money(response['low24h'], 'USD'),
            'last': Money(response['lastPrice'], 'USD'),
            'volume': Money(response['volume24h'], 'BTC')
        }

    def _get_orderbook_from_api_req(self, verify=True):
        return self.req(
            'get',
            '/markets/XBTUSD/order_book',
            no_auth=True,
            verify=verify,
        )

    def place_order_req(self, mode, volume, price=None, order_type=order_types.LIMIT_ORDER):
        side = self._order_mode_from_const(mode)

        if price.currency != 'USD':
            raise ValueError('price must be in USD')
        if volume.currency != 'BTC':
            raise ValueError('volume must be in BTC')

        # Truncate the volume instead of rounding it because it's better# to trade too
        # little than too much.
        volume = volume.round_to_decimal_places(2, rounding=cdecimal.ROUND_DOWN)

        volume_str = '%.2f' % volume.amount
        price_str = '%.2f' % price.amount

        payload = {
            'type': 'limit',
            'currency': 'XBT',
            'side': side,
            'amount': volume_str,
            'price': price_str,
            'instrument': 'XBTUSD'
        }

        return self.req(
            'post',
            '/wallets/%s/orders/' % self.wallet_id,
            data=payload,
        )

    def place_order_resp(self, req):
        response = self.resp(req)

        try:
            order_id = str(response['id'])
            return {'success': True, 'order_id': order_id}
        except KeyError:
            raise exceptions.ExchangeAPIErrorException(
                self,
                'response does not contain an order id',
            )

    def get_open_orders_req(self):
        return self.all_orders_req(status='open')

    def get_open_orders_resp(self, req):
        open_orders = self.all_orders_resp(req)

        for o in open_orders:
            del o['status']

        return open_orders

    def get_order_details(self, order_id):
        req = self.get_order_details_req()
        return self.get_order_details_resp(req, order_id)

    def get_order_details_req(self):
        return self.get_multi_order_details_req()

    def get_order_details_resp(self, req, order_id):
        return self.get_multi_order_details_resp(req, [order_id])[order_id]

    def get_multi_order_details(self, order_ids):
        req = self.get_multi_order_details_req()
        return self.get_multi_order_details_resp(req, order_ids)

    def get_multi_order_details_req(self):
        return self.trades_for_orders_req()

    def get_multi_order_details_resp(self, req, order_ids):
        # This is modeled after Bitstamp, where we get the order details from the
        # trades endpoint directly. The caveat is that order_details will only work
        # for the most recent 50 trades. Since we are always accounting trades right
        # after they happen, this should be ok (and also affects Bitstamp).
        order_ids = [str(o) for o in order_ids]

        multi_trades = self.trades_for_orders_resp(req, order_ids)
        data = {}

        for order_id in order_ids:
            total_usd = Money('0', 'USD')
            total_btc = Money('0', 'BTC')

            our_trades = []
            our_type = None

            if order_id in multi_trades:
                trades = multi_trades[order_id]

                for t in trades:
                    assert(t['currency1'] == 'XBT')
                    btc_amount = Money(t['currency1Amount'], 'BTC')

                    assert(t['currency2'] == 'USD')
                    usd_amount = Money(t['currency2Amount'], 'USD')

                    # This might also come back as XBT, but since ItBit has 0-fee
                    # trading right now, I can't tell.
                    assert(t['commissionCurrency'] == 'USD')
                    fee = Money(t['commissionPaid'], 'USD')

                    total_usd += usd_amount
                    total_btc += btc_amount

                    our_type = self._order_mode_to_const(t['direction'])

                    our_trades.append({
                        'time': parse(t['timestamp']).epoch,
                        'trade_id': None,
                        'fee': fee,
                        'btc': btc_amount,
                        'fiat': usd_amount,
                    })

            time_created = None

            if our_trades:
                time_created = min([t['time'] for t in our_trades])

            data[order_id] = {
                'time_created': time_created,
                'type': our_type,
                'btc_total': total_btc,
                'fiat_total': total_usd,
                'trades': our_trades
            }

        return data

    def cancel_order_req(self, order_id):
        return self.req(
            'delete',
            '/wallets/%s/orders/%s' % (self.wallet_id, order_id),
        )

    def cancel_order_resp(self, req):
        # In the success case, no response is given but we need to call resp() so it
        # can catch any error cases.
        response = self.resp(req)  # noqa
        return {'success': True}

    def withdraw_crypto_req(self, address, volume):
        if not isinstance(address, basestring):
            raise TypeError('Withdrawal address must be a string')

        if not isinstance(volume, Money) or volume.currency != self.volume_currency:
            raise TypeError('Withdrawal volume must be in %s' % self.volume_currency)

        volume_str = '%.8f' % volume.amount

        payload = {
            'currency': 'XBT',
            'amount': volume_str,
            'address': address,
        }

        return self.req(
            'post',
            '/wallets/%s/cryptocurrency_withdrawals' % self.wallet_id,
            data=payload,
        )

    def withdraw_crypto_resp(self, req):
        response = self.resp(req)
        return {'success': True, 'exchange_withdrawal_id': response['withdrawalId']}

    def get_order_audit_data(self, skip_recent=0, page=1):
        """
        Returns an OrderedDict of order ids mapped to their filled volume (only include
        orders that have some trades).
        Dropped the skip_recent flag because we don't seem to be using it anywhere.
        """
        if skip_recent != 0:
            raise ValueEror('skip_recent is deprecated')

        orders = OrderedDict()
        trades_to_audit = self.all_trades(page=page)

        for trade in trades_to_audit:
            order_id = str(trade['orderId'])

            assert(trade['currency1'] == 'XBT')
            trade_amount = abs(Money(trade['currency1Amount'], 'BTC'))

            try:
                orders[order_id] += trade_amount
            except KeyError:
                orders[order_id] = trade_amount

        # Remove the oldest 2 orders, because its trades might be wrapped around a
        # page gap and this would give us an innacurate volume_filled number.
        # We need to remove 2 because there could be an ask and a bid.
        try:
            orders.popitem()
            orders.popitem()
        except KeyError:
            pass

        return orders

    def fiat_deposit_fee(self, deposit_amount):
        return Money('5', 'USD')

    def fiat_withdrawal_fee(self, withdrawal_amount):
        """
        Itbit fee is from their documentation, and an extra $15 is being charged to us
        before it shows up in our bank account (as of the September 2016), so I assume
        that's an intermediary fee.

        The fee should be a flat $50 on withdrawals > $10k, but we'll see.
        """
        fee = Money('0', 'USD')

        if withdrawal_amount < Money('10,000', 'USD'):
            itbit_fee = Money('15', 'USD')
            intermediary_fee = Money('15', 'USD')

            fee = itbit_fee + intermediary_fee
        else:
            fee = Money('50', 'USD')

        return fee



from scipy.stats import pearsonr

def xcorr_coefficent(xs, ys, maxlags=None):
    """
    Calculate the correlation coefficient of two series' at different lags.
    """

    coefficients = []
    pvalues = []

    for i in range(2, len(xs)):
        this_x = xs[0:i]
        this_y = ys[len(ys) - i:]

        coeff, pvalue = pearsonr(this_x, this_y)

        coefficients.append(coeff)
        pvalues.append(pvalue)

    for i in range(0, len(xs)):
        this_x = xs[i:]
        this_y = ys[0:len(ys) - i]

        coeff, pvalue = pearsonr(this_x, this_y)

        coefficients.append(coeff)
        pvalues.append(pvalue)

    if maxlags:
        mid = len(coefficients)/2
        coefficients = coefficients[mid - maxlags:mid + maxlags]
        pvalues = pvalues[mid - maxlags:mid + maxlags]

    return coefficients, pvalues


def xcorr_coefficent_2(xs, ys, maxlags=1):
    """
    Calculate the correlation coefficient of two series' at different lags using a
    faster method.
    """

    coefficients = []
    pvalues = []

    for lag in range(-maxlags, 0):
        lag = abs(lag)
        this_x = xs[0:len(xs) - lag]
        this_y = ys[lag:]

        coeff, pvalue = pearsonr(this_x, this_y)

        coefficients.append(coeff)
        pvalues.append(pvalue)

    for lag in range(0, maxlags + 1):
        this_x = xs[lag:]
        this_y = ys[0:len(ys) - lag]

        coeff, pvalue = pearsonr(this_x, this_y)

        coefficients.append(coeff)
        pvalues.append(pvalue)

    return coefficients, pvalues



import pyximport; pyximport.install()
from cdecimal import Decimal
import inspect
import os
import signal
import sys
import time

from delorean import Delorean
import importlib
import termcolor as tc

from gryphon.execution.strategies.base import Strategy
from gryphon.execution.harness.harness import Harness
from gryphon.execution.lib import tick_profiling
from gryphon.execution.lib.auditing import AuditException
from gryphon.execution.lib.heartbeat import Heartbeat
from gryphon.execution.lib.sentry import Sentry
from gryphon.lib import session
from gryphon.lib.logger import get_logger
from gryphon.lib.models.event import EventRecorder
from gryphon.lib.models.datum import DatumRecorder
from gryphon.lib.util.time import humanize_seconds

logger = get_logger(__name__)

warm_shutdown_flag = False
restart_flag = False

BACKOFF_SECONDS_START = 10
BACKOFF_MULTIPLIER = 2



class NoStrategyClassFoundInModuleException(Exception):
    pass


def trigger_warm_shutdown(signal, frame):
    global warm_shutdown_flag
    logger.info(tc.colored('Setting Warm Shutdown Flag', 'red'))
    warm_shutdown_flag = True


def trigger_restart(signal, frame):
    global restart_flag
    logger.info(tc.colored('Setting Restart Flag', 'red'))
    restart_flag = True
    trigger_warm_shutdown(signal, frame)


signal.signal(signal.SIGINT, trigger_warm_shutdown)
signal.signal(signal.SIGHUP, trigger_restart)


def warm_shutdown(harness, db, sentry, execute):
    logger.info(tc.colored('Initiating Warm Shutdown', 'red'))

    try:
        if execute:
            harness.wind_down()

        logger.info(tc.colored('Warm Shutdown Complete', 'green'))
    except Exception as e:
        sentry.captureException()

        logger.exception(tc.colored(
            '[%s] %s' % (e.__class__.__name__, e.message),
            'red',
        ))

        logger.info(tc.colored(
            'Exception during warm shutdown, forcing shutdown',
            'red',
        ))


def restart():
    """
    execv replaces the current process with a new one which will reload the entire
    context (conf/code)--in this case, a new run of the exact same process. It abruptly
    ends the current process, so db connections, etc must be closed before running this.
    """

    logger.info(tc.colored('Restarting bot with new code/conf\n', 'green'))
    os.execv(sys.executable, [sys.executable] + sys.argv)


def gentle_sleep(seconds):
    """
    warm_shutdown_flag is triggered by ctrl-c. If the harness is in a sleep cycle we
    want to wake up so that we can shut down without a long delay.
    """

    for i in range(0, seconds):
        if warm_shutdown_flag:
            break

        time.sleep(1)


def get_strategy_class_from_module(module):
    """
    Get the strategy class from a module according to a simple ruleset.

    This ruleset is: return the first class defined in the module that is a subclass of
    the Strategy base class, and which case-insensitive matches the module name
    (filename) when any underscores are removed from the module name. This allows us to
    preserve the python convention of using CamelCase in class names and lower_case for
    module names.

    e.g.
        'simple_market_making.pyx' will match class 'SimpleMarketMaking'.
        'geminicoinbasearb.pyx' will match 'GeminCoinbaseArb'.
    """

    expected_strat_name = module.__name__.replace('_', '')

    if '.' in expected_strat_name:  # Probably a module path.
        expected_strat_name = expected_strat_name[expected_strat_name.rfind('.') + 1:]

    for x in dir(module):
        if x.lower() == expected_strat_name.lower():
            cls = getattr(module, x)

            if inspect.isclass(cls) and cls != Strategy and issubclass(cls, Strategy):
                return cls

    raise NoStrategyClassFoundInModuleException()


def get_builtin_strategy_class(strategy_name):
    """
    Load a strategy that ships with the Gryphon library.
    """

    path = 'gryphon.execution.strategies.builtin.%s' % strategy_name.lower()

    module = importlib.import_module(path)

    strategy_class = get_strategy_class_from_module(module)

    return strategy_class


def get_strategy_class_from_filepath(strategy_path):
    """
    Load a strategy that the library user has written and is specified as a filepath
    from the current working directory to a .pyx file.
    """

    module_path = strategy_path.replace('/', '.').replace('.pyx', '')

    # Since in this case we're importing a file outside of the library, we have to
    # add our current directory to the python path.
    sys.path.append(os.getcwd())

    module = importlib.import_module(module_path)
    strategy_class = get_strategy_class_from_module(module)

    return strategy_class


def get_strategy_class(strategy_path, builtin=False):
    if builtin is True:
        return get_builtin_strategy_class(strategy_path)
    else:
        return get_strategy_class_from_filepath(strategy_path)


def exception_retry_loop(harness, sentry, db):
    success = False
    backoff_seconds = BACKOFF_SECONDS_START

    while not success and not warm_shutdown_flag:
        success = handle_exception(harness, sentry)

        if not success:
            logger.info(tc.colored(
                'handle_exception failed, backing off for %s' % (
                humanize_seconds(backoff_seconds)),
                'red',
            ))

            gentle_sleep(backoff_seconds)
            backoff_seconds *= BACKOFF_MULTIPLIER
        else:
            logger.info(tc.colored(
                'handle_exception succeeded, full speed ahead',
                'green',
            ))

            return

        # This is necessary to flush local db cache and pick up changes other processes 
        # have made. For example, if this is a balance mismatch which was fixed by the
        # withdrawal tool.
        session.commit_mysql_session(db)


def handle_exception(harness, sentry):
    """
    Since much of our recovery code is in the audits, trying to handle an exception
    while we're not auditing is poorly defined. Possibly we should audit here so long
    as the strategy is running with db backing here, even if they are turned off in
    the configuration.
    """
    try:
        if harness.audit is True:
            harness.full_audit()

        return True
    except Exception as e:
        sentry.captureException()
        logger.exception(tc.colored(
            '[%s] %s' % (e.__class__.__name__, e.message),
            'red',
        ))

        return False


def live_run(configuration):
    strategy_name = configuration['platform']['strategy']
    is_builtin_strategy = configuration['platform']['builtin']
    execute = configuration['platform']['execute']

    logger.info('live_run(%s, %s)' % (strategy_name, execute))

    db = session.get_a_trading_db_mysql_session()

    harness = Harness(db, configuration)

    strategy_class = get_strategy_class(strategy_name, is_builtin_strategy)
    strategy = strategy_class(db, harness, configuration['strategy'])
    strategy.set_up()

    harness.strategy = strategy

    if execute:
        EventRecorder().create(db=db)
        DatumRecorder().create(db=db)
    else:
        EventRecorder().create()
        DatumRecorder().create()

    sentry = Sentry(configuration['platform']['sentry'])
    heartbeat = Heartbeat(configuration['platform']['heartbeat'])

    try:
        tick_count = 0

        while True:
            try:
                tick_start = Delorean().epoch
                print '\n\n%s' % strategy.name

                if warm_shutdown_flag:
                    return  # This takes us into the finally block.

                # Initial audit. This is done inside the main loop so that our robust
                # exception catching kicks in on initial audit failures.
                if harness.audit is True and tick_count == 0:
                    # We try a fast audit (no wind down) since the bots usually start
                    # from a clean slate.
                    try:
                        harness.full_audit(wind_down=False)
                    except AuditException:
                        logger.info(
                            'Bot was not cleanly shut down, winding down and auditing',
                        )

                        harness.full_audit(wind_down=True)

                # Regular audits.
                if (harness.audit is True
                        and tick_count > 0
                        and tick_count % harness.audit_tick == 0):
                    harness.full_audit()
                else:
                    harness.tick()

                    harness.post_tick(tick_count)

                tick_profiling.record_tick_data(tick_start, strategy.name)
                tick_profiling.record_tick_block_data(
                    strategy,
                    tick_count,
                    strategy.name,
                )

                heartbeat.heartbeat(strategy.name)

            except Exception as e:
                sentry.captureException()

                logger.exception(tc.colored(
                    '[%s] %s' % (e.__class__.__name__, e.message),
                    'red',
                ))

                exception_retry_loop(harness, sentry, db)
            finally:
                session.commit_mysql_session(db)
                tick_count += 1

            if harness.strategy_complete() is True:
                break
            else:
                gentle_sleep(harness.sleep_time_to_next_tick())
    finally:
        warm_shutdown(harness, db, sentry, execute)
        session.commit_mysql_session(db)
        db.remove()

        if restart_flag:
            restart()




import numpy as np
import time

sampleData = open('sampleData.txt','r').read()
splitData = sampleData.split('\n')

date,closep,highp,lowp,openp,volume = np.loadtxt(splitData,
                                                 delimiter=',',
                                                 unpack=True)

def cog(dates,data,tf):

    COG = []
    x = tf
    while x < len(dates):
        consider = data[x-tf:x]
        #print consider
        #print len(consider)

        multipliers = range(1,tf+1)

        topFrac = 0
        botFrac = 0

        reversedOrder = reversed(consider)

        ordered = []
        for eachItem in reversedOrder:
            ordered.append(eachItem)

        #print ordered
        
        for eachM in multipliers:
            addMe = eachM*ordered[eachM-1]

            addMe2 = ordered[eachM-1]
            #print addMe
            topFrac+=addMe
            botFrac+=addMe2

        CeOfGr = -(topFrac/botFrac)

        #print CeOfGr
        COG.append(CeOfGr)
            
            
        #return dates[tf:]    

        #time.sleep()
        x+=1


    return dates[tf:],COG

cog(date,closep,10)



"""
This library helps determine the best option to close an open position between different
exchanges and order types.
"""

from gryphon.lib.exchange import exchange_factory
from gryphon.lib.exchange.base import Exchange
from gryphon.lib.gryphonfury import fees as fees_lib
from gryphon.lib.gryphonfury import positions
from gryphon.lib.gryphonfury import revenue as revenue_lib
from gryphon.lib.metrics import quote as quote_lib
from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook
from gryphon.lib.models.order import Order
from gryphon.lib.money import Money


def get_pl_from_market_order(position_trades, orderbook, exchange_name):
    """
    Given our current position and an exchange orderbook, what would our realized p&l
    be if we closed that position right now with a market order?
    """
    pl = Money('0', 'USD')

    open_position = positions.position_delta(position_trades)
    unmatched_fees = revenue_lib.all_fees(position_trades)[0].to('USD')

    fiat_position = open_position['fiat'].to('USD')
    btc_position = open_position['BTC']

    value_of_btc = Money(0, 'BTC')

    if btc_position > 0:
        value_of_btc = quote_lib.price_quote_from_orderbook(
            orderbook,
            Order.ASK,
            btc_position,
        )['total_price']
    else:
        value_of_btc = quote_lib.price_quote_from_orderbook(
            orderbook,
            Order.BID,
            btc_position,
        )['total_price']

    revenue = fiat_position + unmatched_fees + value_of_btc.to('USD')

    exchange = exchange_factory.make_exchange_from_key(exchange_name)
    new_fee = exchange.market_order_fee * abs(value_of_btc.to('USD'))

    fees = unmatched_fees + new_fee

    return revenue, fees


def get_pl_from_open_orders(position_trades, exchange_name, open_orders):
    """
    Given our current position, and the open orders on an exchange, what would our
    realized p&l be if a market order came in and consumed our order entirely?

    Details: we naively assume that our order is sized such that it would completely
    close our position. If there is no open order on that exchange, we return None.
    """

    bids = open_orders['bids']
    asks = open_orders['asks']

    pl = Money('0', 'USD')

    open_position = positions.position_delta(position_trades)
    unmatched_fees = revenue_lib.all_fees(position_trades)[0].to('USD')

    fiat_position = open_position['fiat'].to('USD')
    btc_position = open_position['BTC']

    value_of_btc = Money(0, 'USD')

    if btc_position > 0 and len(asks) > 0:
        close_order = asks[0]  # Naively assume the first order is what we care about.

        close_price = Money(close_order[0], 'USD')

        value_of_btc = close_price * btc_position.amount
    elif btc_position <= 0 and len(bids) > 0:
        close_order = bids[0]

        close_price = Money(close_order[0], 'USD')

        value_of_btc = close_price * btc_position.amount
    else:
        return Money(0, 'USD'), Money(0, 'USD')

    revenue = fiat_position + unmatched_fees + value_of_btc.to('USD')

    exchange = exchange_factory.make_exchange_from_key(exchange_name)
    new_fee = exchange.limit_order_fee * abs(value_of_btc.to('USD'))

    fees = unmatched_fees + new_fee

    return revenue, fees


def get_pl_from_limit_order(position_trades, orderbook, exchange_name):
    """
    Given our current position and an exchange orderbook, what would our realized p&l
    be on that position if we were the top order on this exchange, in the amount and
    side that would exactly close our position, and a market order came through and took
    the whole order?
    """
    pl = Money('0', 'USD')

    open_position = positions.position_delta(position_trades)
    unmatched_fees = revenue_lib.all_fees(position_trades)[0].to('USD')

    fiat_position = open_position['fiat'].to('USD')
    btc_position = open_position['BTC']

    value_of_btc = Money(0, 'USD')

    if btc_position > 0:
        # Then we're in a long position, we want to sell, so place an order in front
        # of the top ask.
        top_ask = orderbook['asks'][0]

        close_price = top_ask.price - Money('0.01', top_ask.price.currency)

        value_of_btc = close_price * btc_position.amount
    else:
        # Then we're in a long position, we want to sell, so place an order in front
        # of the top bid.
        top_bid = orderbook['bids'][0]

        close_price = top_bid.price + Money('0.01', top_bid.price.currency)

        value_of_btc = close_price * btc_position.amount

    revenue = fiat_position + unmatched_fees + value_of_btc.to('USD')

    exchange = exchange_factory.make_exchange_from_key(exchange_name)
    new_fee = exchange.market_order_fee * abs(value_of_btc.to('USD'))

    fees = unmatched_fees + new_fee

    return revenue, fees




from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class BitfinexOrderbook(OrderbookPoller):
    """
    2018/01/17 This endpoint currently has a 60/minute rate limit.
    """

    def __init__(self):
        self.exchange_name = u'BITFINEX_BTC_USD'
        self.poll_time = 3
        self.orderbook_depth = 50000
        self.url = 'https://api.bitfinex.com/v1/book/btcusd?limit_bids=%s&limit_asks=%s' % (self.orderbook_depth, self.orderbook_depth)

    def parse_order(self, order):
        """
        API response format:
        {"bids":[{"price":"405.02","amount":"0.001","timestamp":"1411774902.0"}, ...
        """
        return [order['price'], order['amount'], order['timestamp']]



# -*- coding: utf-8 -*-
import os
import requests
import random
from delorean import epoch, Delorean
from collections import OrderedDict
from datetime import datetime, timedelta
from gryphon.lib.money import Money
from gryphon.lib.bitcoinwisdom import BitcoinWisdom


class TwistedBitcoinWisdom(BitcoinWisdom):
    
    from twisted.internet import defer
    from twisted.python import log

    @defer.inlineCallbacks
    def volume_in_period(self, start_date, end_date):
        from twisted.internet import defer
        total_volume = Money(0, 'BTC')
        periods = yield self.period(step=self.determine_step(start_date, end_date))
        for k,v in periods.iteritems():
            t=Delorean(k, 'UTC').datetime
            vol=v
            if t >= start_date and t < end_date:
                total_volume += vol
        defer.returnValue(total_volume)
    
    @defer.inlineCallbacks
    def period(self, step='1d', sid=''): 
        from twisted.internet import defer
        points = yield self.req(self.create_url(step))
        if not points:
            defer.returnValue(OrderedDict())
        point_hash = OrderedDict()
        for p in points:
            point_hash[epoch(p[0]).naive] = Money(str(p[7]), 'BTC')
        defer.returnValue(point_hash)
    
    @defer.inlineCallbacks
    def req(self, url):
        import treq
        from twisted.internet import defer
        response = yield treq.get('%s&nonce=5408178796808' % url)
        json_response = yield treq.json_content(response)
        a = yield json_response
        defer.returnValue(a)
 
    



from autobahn.twisted.websocket import WebSocketClientProtocol
from twisted.internet import reactor
from twisted.python import log


class EmeraldWebSocketClientProtocol(WebSocketClientProtocol):
    def onConnect(self, response):
        log.msg("Web Socket Server connected: {0}".format(response.peer))

    def onOpen(self):
        raise NotImplementedError

    def onMessage(self, payload, isBinary):
        raise NotImplementedError

    def onClose(self, wasClean, code, reason):
        log.msg(
            "WebSocket connection closed: %s : Code %s : wasClean:%s. Restarting." % (
            reason,
            code,
            wasClean,
        ))

        reactor.callLater(2, self.start)



"""
This file tests the Balance class, which also tests for us the Position and Target
classes, since they are just clones of Balance.
"""
import pyximport; pyximport.install()

import unittest
import sure

from gryphon.lib.money import Money
from gryphon.lib.models.exchange import Balance


class TestBalance(unittest.TestCase):
    def setUp(self):
        self.balance = Balance({
            'USD': Money("100", "USD"),
            'BTC': Money("1", "BTC")
        })

    def tearDown(self):
        pass

    def test_empty(self):
        balance = Balance()
        balance['BTC'].should.equal(Money("0", "BTC"))

    def test_add_balance(self):
        balance2 = Balance({
            'USD': Money("50", "USD"),
            'BTC': Money("2", "BTC")
        })
        new_balance = self.balance + balance2
        new_balance['USD'].should.equal(Money("150", "USD"))
        new_balance['BTC'].should.equal(Money("3", "BTC"))

    def test_add_money(self):
        new_balance = self.balance + Money("0.2", "BTC")
        new_balance['USD'].should.equal(Money("100", "USD"))
        new_balance['BTC'].should.equal(Money("1.2", "BTC"))

    def test_negate(self):
        neg = -self.balance
        neg['USD'].should.equal(Money("-100", "USD"))
        neg['BTC'].should.equal(Money("-1", "BTC"))

    def test_fiat(self):
        fiat = self.balance.fiat()
        fiat.should.equal(Money("100", "USD"))

    def test_add_result_mutability(self):
        m = Money(2, 'BTC')
        new_balance = self.balance + m #  => USD 100, BTC 3
        new_balance['USD'].should.equal(Money("100", "USD"))

        self.balance['USD'].amount += 10 # USD 110

        # Changing self.balance shouldn't affect new_balance
        new_balance['USD'].should.equal(Money("100", "USD"))



# -*- coding: utf-8 -*-
import logging

from delorean import Delorean
import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.dashboards.handlers.mixins.start_and_end_time import StartAndEndTimeMixin
import gryphon.dashboards.util.balances as balance_util
from gryphon.dashboards.util import queries
from gryphon.lib.exchange.exchange_factory import all_exchanges
from gryphon.lib.models.transaction import Transaction


logger = logging.getLogger(__name__)


class AllBalancesHandler(AdminBaseHandler, StartAndEndTimeMixin):
    @tornado.web.authenticated
    def get(self, month=None):
        start_time, end_time = self.get_start_time_and_end_time()

        start_time = queries.strip_datetime_for_db_operations(start_time)
        end_time = queries.strip_datetime_for_db_operations(end_time)

        start_timestamp = Delorean(start_time, 'UTC').epoch * 1000
        end_timestamp = Delorean(end_time, 'UTC').epoch * 1000

        usd_balances_series = {}
        btc_balances_series = {}

        for exchange in all_exchanges():
            usd_balances, btc_balances = balance_util.get_balance_time_series_for_exchange(self.trading_db, exchange.name, start_time, end_time)

            usd_balances_series[exchange.name] = usd_balances
            btc_balances_series[exchange.name] = btc_balances

        withdrawals = self.trading_db\
            .query(Transaction)\
            .filter(Transaction.time_created >= start_time)\
            .filter(Transaction.time_created < end_time)\
            .filter(Transaction._amount_currency == 'BTC')\
            .filter(Transaction.transaction_type == 'WITHDRAWL')\
            .all()

        deposits = self.trading_db\
            .query(Transaction)\
            .filter(Transaction.time_created >= start_time)\
            .filter(Transaction.time_created < end_time)\
            .filter(Transaction._amount_currency == 'BTC')\
            .filter(Transaction.transaction_type == 'DEPOSIT')\
            .all()

        bitcoin_withdrawals = [[int(Delorean(movement.time_created, "UTC").epoch) * 1000, float(movement.amount.amount)] for movement in withdrawals]

        bitcoin_deposits = [[int(Delorean(movement.time_created, "UTC").epoch) * 1000, float(movement.amount.amount)] for movement in deposits]

        self.render_template(
            'balances.html',
            args={
                'usd_balances_series': usd_balances_series,
                'btc_balances_series': btc_balances_series,
                'bitcoin_withdrawals': bitcoin_withdrawals,
                'bitcoin_deposits': bitcoin_deposits,
                'start_timestamp': start_timestamp,
                'end_timestamp': end_timestamp,
            },
        )




from gryphon.data_service.pollers.orderbook.orderbook_poller import OrderbookPoller


class ItbitOrderbook(OrderbookPoller):
    def __init__(self):
        self.exchange_name = u'ITBIT_BTC_USD'
        self.url = 'https://api.itbit.com/v1/markets/XBTUSD/order_book'
        self.poll_time = 2

    # API response format:
    # {"bids":[["373.51","1.6463"], ...

    def parse_order(self, order):
        return [order[0], order[1], '']



# -*- coding: utf-8 -*-
from datetime import timedelta
import logging

from delorean import Delorean
import tornado.web

from gryphon.dashboards.handlers.admin_base import AdminBaseHandler
from gryphon.lib.exchange import exchange_factory
from gryphon.lib.models.order import Order
from gryphon.lib.models.exchange import Exchange as ExchangeData
from gryphon.lib.models.exchange import Balance
from gryphon.lib.models.transaction import Transaction
from gryphon.lib.money import Money


logger = logging.getLogger(__name__)

BANK_ACCOUNT_HIGHLIGHT_THRESHOLD = 30000


class GryphonStatusHandler(AdminBaseHandler):
    @tornado.web.authenticated
    def get(self):
        exchanges = exchange_factory.get_all_initialized_exchange_wrappers(
            self.trading_db,
        )

        exchange_info = self.get_exchange_info(exchanges)
        system_balances, total_fiat = self.get_system_balances(exchanges)
        bank_accounts = self.get_trading_bank_accounts()
        in_transit_fiat_txs, in_transit_btc_txs = self.get_in_transit_transactions()
        recent_transactions = self.get_recent_transactions()
        net_flows = self.get_daily_net_transaction_flows(exchanges)

        self.render_template(
            'status.html',
            args={
                'all_exchanges': exchange_info,
                'system_balances': system_balances,
                'bank_accounts': bank_accounts,
                'total_fiat': total_fiat,
                'in_transit_fiat_txs': in_transit_fiat_txs,
                'in_transit_btc_txs': in_transit_btc_txs,
                'recent_transactions': recent_transactions,
                'net_flows': net_flows,
            },
        )

    def get_daily_net_transaction_flows(self, exchanges):
        flows = []
        one_day_ago = Delorean().last_day(1).naive

        for exchange in exchanges:
            exchange_data = exchange.exchange_account_db_object(self.trading_db)

            exchange_txs = exchange_data.transactions\
                .filter(Transaction._amount_currency == 'BTC')\
                .filter(Transaction.time_created > one_day_ago)\
                .filter(Transaction.transaction_status == 'COMPLETED')\
                .order_by(Transaction.time_created.desc())\
                .all()

            deposit_total = sum([
                round(tx.amount.amount, 0)
                for tx in exchange_txs if tx.transaction_type == Transaction.DEPOSIT
            ])

            withdrawal_total = sum([
                round(tx.amount.amount, 0)
                for tx in exchange_txs if tx.transaction_type == Transaction.WITHDRAWL
            ])

            flows.append({
                'exchange_name': exchange.name,
                'withdrawals': withdrawal_total,
                'deposits': deposit_total,
            })

        flows = sorted(
            flows,
            key=lambda flow: flow.get('withdrawals') + flow.get('deposits'),
            reverse=True
        )

        return flows

    def get_recent_transactions(self):
        five_hours_ago = Delorean().naive - timedelta(hours=5)

        txs = self.trading_db.query(Transaction)\
            .filter(Transaction.time_created > five_hours_ago)\
            .filter_by(transaction_status='COMPLETED')\
            .order_by(Transaction.time_created.desc())\
            .join(ExchangeData)\
            .all()

        return txs

    def get_in_transit_transactions(self):
        """
        Query all IN_TRANSIT transactions from the database.

        Returns the fiat tx results, and then the btc ones
        """
        all_txs_query = self.trading_db.query(Transaction)\
            .filter_by(transaction_status='IN_TRANSIT')\
            .join(ExchangeData)\
            .order_by(Transaction.time_created)

        fiat_txs = all_txs_query.filter(Transaction._amount_currency != "BTC").all()
        btc_txs = all_txs_query.filter(Transaction._amount_currency == "BTC").all()

        return fiat_txs, btc_txs

    def get_exchange_info(self, exchanges):
        up_exchanges = self.get_up_exchanges()

        exchange_info = []

        for exchange in exchanges:
            exchange_data = exchange.exchange_account_db_object(self.trading_db)
            exchange_dict = {
                'name': exchange.name,
                'balance': exchange_data.balance,
                'is_up': exchange.name in up_exchanges.keys(),
                'up_since': up_exchanges.get(exchange.name),
            }

            exchange_info.append(exchange_dict)

        exchange_info = sorted(exchange_info, key=lambda e: e.get('is_up') is False)

        return exchange_info

    def get_system_balances(self, exchanges):
        system_balance = Balance()

        for e in exchanges:
            system_balance += e.exchange_account_db_object(self.trading_db).balance

        total_fiat = sum([
            balance.to("USD") for currency, balance in system_balance.iteritems()
            if currency not in Money.CRYPTO_CURRENCIES
        ])

        return system_balance, total_fiat

    def get_up_exchanges(self):
        now = Delorean().naive
        ten_minutes_ago = now - timedelta(minutes=10)

        orders = self.trading_db\
            .query(Order)\
            .filter(Order.time_created > ten_minutes_ago)\
            .order_by(Order.time_created.asc())\
            .all()

        up_exchanges = {}

        for order in orders:
            up_exchanges[order._exchange_name] = order.time_created

        # TODO: convert these into "minutes ago".
        for key in up_exchanges.keys():
            up_exchanges[key] = now - up_exchanges[key]

        return up_exchanges

    def get_trading_bank_accounts(self):
        trading_bank_acount_keys = ['BMO_USD', 'BMO_CAD']

        bank_account_infos = []

        for key in trading_bank_acount_keys:
            account = exchange_factory.make_exchange_data_from_key(
                key,
                self.trading_db,
            )

            account_info = {
                'name': account.name,
                'balance': account.balance.fiat(),
                'highlight': account.balance.fiat() > BANK_ACCOUNT_HIGHLIGHT_THRESHOLD,
            }
            bank_account_infos.append(account_info)

        return bank_account_infos




"""
This script allows pip users to run migrations on the databases they use with gryphon.
Gryphon uses alembic to manage migrations.

Usage:
    gryphon-execute script run_migrations
        --database [GDS | TRADING | DASHBOARD]
"""

import pyximport; pyximport.install()

import logging
import os
from pkg_resources import resource_filename

from alembic.config import Config
from alembic import command

from gryphon.lib import session

logger = logging.getLogger(__name__)


GDS_CONFIG_RELATIVE_PATH = 'data_service/alembic.ini'
DASHBOARD_CONFIG_RELATIVE_PATH = 'dashboards/alembic.ini'
TRADING_CONFIG_RELATIVE_PATH = 'execution/alembic.ini'

PACKAGE_NAME = 'gryphon'

GDS_DB_NAME = 'gds'
TRADING_DB_NAME = 'trading'
DASHBOARD_DB_NAME = 'dashboard'
DATABASE_NAMES = [GDS_DB_NAME, TRADING_DB_NAME, DASHBOARD_DB_NAME]

NO_EXECUTE_MESSAGE = 'Not running migration because execute == False'

UNKNOWN_DB_MESSAGE = 'There is no database %s associated with the gryphon-framework.'


def run_migrations(target_db):
    location = None

    if target_db == GDS_DB_NAME:
        location = resource_filename(PACKAGE_NAME, GDS_CONFIG_RELATIVE_PATH)
    elif target_db == DASHBOARD_DB_NAME:
        location = resource_filename(PACKAGE_NAME, DASHBOARD_CONFIG_RELATIVE_PATH)
    elif target_db == TRADING_DB_NAME:
        location = resource_filename(PACKAGE_NAME, TRADING_CONFIG_RELATIVE_PATH)
    
    alembic_cfg = Config(location)
    command.upgrade(alembic_cfg, 'head')


def main(script_arguments, execute):
    target_db = script_arguments['database']

    target_db = target_db.lower()

    if target_db in DATABASE_NAMES:
        logger.info('Migrating the %s database' % target_db)
    else:
        logger.info(UNKNOWN_DB_MESSAGE % target_db)
        return

    if execute is True:
        run_migrations(target_db)
    else:
        logger.info('Not running migration because execute == False')




"""
Somewhere between 0.3 and 1.0, Delorean changed the behaviour of their 'parse' function
so that it no longer defaults to parsing strings by iso 8601. I have no idea why, it may
have to do with downstream changes in python-dateutil. In any case, this function hacks
the delorea.parse() function back into it's old behaviour.
"""

import delorean


def parse(*args, **kwargs):
    return delorean.parse(*args, dayfirst=False, **kwargs)



#November 11th 2013:
open1 = 1286.50
high1 = 1287.70
low1  = 1278.50
close1= 1281.70

#November 12th 2013:
open2 = 1281.80
high2 = 1284.50
low2  = 1260.70
close2= 1266.70

limitMove = 75

def SwingIndex(O1,O2,H1,H2,L1,L2,C1,C2,LM):

    def calc_R(H2,C1,L2,O1,LM):
        x = H2-C1
        y = L2-C1
        z = H2-L2
        print x
        print y
        print z

        if z < x > y:
            print 'x wins!'
            R = (H2-C1)-(.5*(L2-C1))+(.25*(C1-O1))
            print R
            return R
        elif x < y > z:
            print 'y wins!'
            R = (L2-C1)-(.5*(H2-C1))+(.25*(C1-O1))
            print R
            return R

        elif x < z > y:
            print 'z wins!'
            R = (H2-L2)+(.25*(C1-O1))
            print R
            return R


    def calc_K(H2,L2,C1):
        x = H2-C1
        y = L2-C1

        if x > y:
            K=x
            print K
            return K
        elif x < y:
            K=y
            print K
            return K

    L = LM
    R = calc_R(H2,C1,L2,O1,LM)
    K = calc_K(H2,L2,C1)

    SwIn = 50*((C2-C1+(.5*(C2-O2))+(.25*(C1-O1)))/R)*(K/L)
    print '###'
    print SwIn
    



    
SwingIndex(open1,open2,high1,high2,low1,low2,close1,close2,limitMove)



# -*- coding: utf-8 -*-
from gryphon.data_service.pollers.trades.trades_poller import TradesPoller
from gryphon.lib.money import Money


class BitfinexTrades(TradesPoller):
    """
    Bitfinex's trades endpoint currently has a rate limit of 45/minute.

    This endpoint operates by returning the last n trades when it is called, with
    n=50 as the default. There is also an option to only return trades after a given
    timestamp. Both of these pieces of functionality are unimplemented currently.
    """

    def __init__(self):
        self.exchange_name = u'BITFINEX_BTC_USD'
        self.url = 'https://api.bitfinex.com/v1/trades/btcusd'
        self.poll_time = 4

    def parse_trades(self, resp_obj):
        trades = []

        for trade in resp_obj:
            trade = {
                'exchange': self.exchange_name,
                'price': Money(trade['price'], 'USD'),
                'volume': Money(trade['amount'], 'BTC'),
                'timestamp': trade['timestamp'],
                'trade_id': trade['tid']
            }

            trades.append(trade)

        return trades



__version__ = '0.1'

def main():
    """Entry point for the application script"""
    print("Call your main application code here")

def monkeypatch_decimal_to_cdecimal():
    # monkeypatch the "decimal" library in.
    # this is a drop-in replacement for "decimal".
    try:
        import cdecimal
        import sys
        sys.modules['decimal'] = cdecimal
    except ImportError:
        pass

def prepare():
    monkeypatch_decimal_to_cdecimal()



# -*- coding: utf-8 -*-
import logging
import os
import traceback

import sqlalchemy.exc
import tornado.web
from tornadotoad import api
from tornadotoad import my

from gryphon.lib.models.emeraldhavoc.orderbook import Orderbook

logger = logging.getLogger(__name__)

P3P_POLICY = u"""\
CP="Like Facebook, This App does not have a P3P policy -Learn why: http://fb.me/p3p\
"""


class BaseHandler(tornado.web.RequestHandler):
    """
    A class to collect common handler methods - all other handlers should subclass this
    one.
    """
    @property
    def dashboard_db(self):
        return self.application.dashboard_db

    @property
    def trading_db(self):
        return self.application.trading_db

    @property
    def gds_db(self):
        return self.application.gds_db

    @property
    def configuration(self):
        return self.application.configuration

    def send_error(self, status_code, **kwargs):
        if status_code == 403 and not my.log_403:
            return super(BaseHandler, self).send_error(status_code, **kwargs)

        if status_code == 404 and not my.log_404:
            return super(BaseHandler, self).send_error(status_code, **kwargs)

        if status_code == 405 and not my.log_405:
            return super(BaseHandler, self).send_error(status_code, **kwargs)

        tornado_toad = api.TornadoToad()
        exception = kwargs.get('exc_info')

        if exception:
            formatted_arguments = {}

            for key in self.request.arguments.keys():
                formatted_arguments[key] = u','.join(self.request.arguments[key])

            request_data = {
                'url': self.request.full_url(),
                'component': self.__class__.__name__,
                'cgi-data': self.request.headers,
                'params': formatted_arguments,
            }

            tornado_toad.post_notice(exception, request=request_data)

        return super(BaseHandler, self).send_error(status_code, **kwargs)

    def commit_session(self):
        try:
            self.dashboard_db.commit()
        except Exception as e:
            self.dashboard_db.rollback()

            logger.critical(u'BASE - commit needed to roll back.')
            logger.critical(
                u'Transaction needed to be rolled back because of: \n %s' % (
                traceback.format_exc(),
            ))

    def prepare(self):
        if (os.environ.get('APP_ENV') == 'PRODUCTION'
                and self.request.headers.get('X-Forwarded-Proto') != 'https'):
            old_url = self.request.full_url()
            new_url = old_url.replace('http://', 'https://')
            self.redirect(new_url, permanent=True)

            return

        p3p = P3P_POLICY
        self.add_header('Accept-Charset', u'utf-8')
        self.add_header('X-Frame-Options', u'deny')
        self.set_header('P3P', p3p)

    @property
    def should_recalculate_stored_results(self):
        return self.get_argument('recalculate', None) is not None

    def is_gds_connection_active(self):
        """
        Simple function to test if we have a working connection to a GDS instance so
        we don't error out accidentally.
        """
        if self.gds_db is None:
            return False

        try:
            self.gds_db.query(Orderbook.orderbook_id).first()
        except sqlalchemy.exc.OperationalError as error:
            return False

        return True

    def on_finish(self):
        try:
            self.dashboard_db.commit()
        except Exception:
            self.dashboard_db.rollback()

            logger.critical(
                u'Transaction needed to be rolled back because of: \n %s' % (
                traceback.format_exc(),
            ))
        finally:
            self.dashboard_db.remove()

        try:
            self.trading_db.commit()
        except Exception:
            self.trading_db.rollback()

            logger.critical(
                u'Transaction needed to be rolled back because of: \n %s' % (
                traceback.format_exc(),
            ))
        finally:
            self.trading_db.remove()

        if self.gds_db is not None:
            try:
                self.gds_db.commit()
            except Exception:
                self.gds_db.rollback()

                logger.critical(
                    u'TC Transaction needed to be rolled back because of: \n %s' % (
                    traceback.format_exc(),
                ))
            finally:
                self.gds_db.remove()



# -*- coding: utf-8 -*-
from collections import defaultdict
from datetime import datetime, date, timedelta
import json
import os
import uuid

from decimal import *
from delorean import epoch
from sqlalchemy import ForeignKey, Column, Integer, Unicode, DateTime, UnicodeText, Numeric, Index
from sqlalchemy.orm import relationship, backref
from sqlalchemy.ext.declarative import declarative_base

from gryphon.lib.money import Money
from gryphon.lib import gryphon_json_serialize
from gryphon.lib.exchange.exchange_factory import make_exchange_from_key
from gryphon.lib.models.base import Base
from gryphon.lib.models.basic_order import BasicOrder
from gryphon.lib.models.datum import Datum
from gryphon.lib.models.trade import Trade

metadata = Base.metadata


class Order(Base, BasicOrder):
    __tablename__ = 'order'
    
    order_id = Column(Integer, primary_key=True)
    status = Column(Unicode(32), nullable=False)
    order_type = Column(Unicode(64))
    unique_id = Column(Unicode(64), nullable=False)
    exchange_order_id = Column(Unicode(64))
    actor = Column(Unicode(64))
    exchange_rate = Column(Numeric(precision=20, scale=10))
    
    time_created = Column(DateTime, nullable=False)
    time_executed = Column(DateTime)

    _exchange_name = Column('exchange_name', Unicode(64))
    _price = Column('price', Numeric(precision=20, scale=10))
    _price_currency = Column('price_currency', Unicode(64))
    _volume = Column('volume', Numeric(precision=20, scale=10))
    _volume_currency = Column('volume_currency', Unicode(64))
    _fundamental_value = Column('fundamental_value', Numeric(precision=20, scale=10))
    _fundamental_value_currency = Column('fundamental_value_currency', Unicode(3))
    _competitiveness = Column('competitiveness', Numeric(precision=20, scale=10))
    _competitiveness_currency = Column('competitiveness_currency', Unicode(3))
    _spread = Column('spread', Numeric(precision=20, scale=10))
    _spread_currency = Column('spread_currency', Unicode(3))
    
    trades = relationship('Trade', cascade="all,delete-orphan", backref='order')
    datums = relationship("Datum",  backref='order')
    
    __table_args__ = (Index('idx_exchange_order_id_exchange_name', exchange_order_id, _exchange_name),
                      Index('idx_status', status),)
    
    def __init__(self, actor, mode, volume, price, exchange, exchange_order_id):
        self.status = self.OPEN
        self.unique_id = unicode(uuid.uuid4().hex)
        self.time_created = datetime.utcnow()

        assert actor and price and volume and exchange and mode
        self.actor = actor
        self.order_type = mode.upper()
        self.price = price
        self.volume = volume
        self.exchange = exchange
        self.exchange_order_id = exchange_order_id
        self.currency = self.exchange.currency

    def __unicode__(self):
        return u'[ORDER:%s:%s] Status:%s, Price:%s, Volume:%s BTC' % (
            self.order_type, self.exchange.name, self.status, self.price, self.volume)
        
    def __repr__(self):
        return self.to_json()

    def to_json(self):
        return json.dumps({
            'order_id':self.order_id,
            'unique_id':self.unique_id,
            'exchange_order_id':self.exchange_order_id,
            'time_created':unicode(self.time_created),
            'time_executed':unicode(self.time_executed),
            'exchange':self.exchange.name,
            'status':self.status,
            'order_type': self.order_type,
            'price':self.price,
            'volume':self.volume,
            'trades': ["%s @ %s" % (t.volume, t.price) for t in self.trades]
        }, ensure_ascii=False)

    def set_trades(self, trades):
        self.trades = []
        for trade in trades:
            new_trade = Trade(
                self.order_type,
                trade['fiat'],
                trade['fee'],
                trade[self._volume_currency.lower()],
                trade['trade_id'],
                self)

            new_trade.time_created = epoch(trade['time']).datetime

    # returns position change
    def was_eaten(self, order_details):
        old_position = self.calc_position()
        old_position_no_fees = self.calc_position(include_fees=False)

        self.time_executed = datetime.utcnow()

        if order_details['%s_total' % self._volume_currency.lower()] <= 0:
            self.status = Order.CANCELLED
        else:
            self.status = Order.FILLED
            # TODO store the volume filled amount in order?
            self.time_executed = datetime.utcnow()
            self.set_trades(order_details['trades'])

        position_change = self.position_change(old_position)
        position_change_no_fees = self.position_change(
            old_position_no_fees,
            include_fees=False,
        )

        return position_change, position_change_no_fees

    # returns position change
    def was_partially_eaten(self, order_details):
        old_position = self.calc_position()
        old_position_no_fees = self.calc_position(include_fees=False)

        self.set_trades(order_details['trades'])

        position_change = self.position_change(old_position)
        position_change_no_fees = self.position_change(old_position_no_fees, include_fees=False)

        return position_change, position_change_no_fees

    # returns position change
    def reverse(self):
        old_position = self.position
        self.status = Order.REVERSED
        self.set_trades([])
        return self.position_change(old_position)

    @property
    def exchange(self):
        if hasattr(self, '_exchange'):
            return self._exchange
        elif self._exchange_name:
            self._exchange = make_exchange_from_key(self._exchange_name)
            return self._exchange
        else:
            return None
        
    @exchange.setter
    def exchange(self, value):
        self._exchange = value
        self._exchange_name = self._exchange.name
    
    
    @property
    def volume(self):
        if not self._volume:
            return None
        return Money(self._volume, self._volume_currency)

    @volume.setter
    def volume(self, value):
        self._volume = value.amount
        self._volume_currency = value.currency
    
    @property
    def price(self):
        if not self._price:
            return None
        return Money(self._price, self._price_currency)

    @price.setter
    def price(self, value):
        self._price = value.amount
        self._price_currency = value.currency

    @property
    def competitiveness(self):
        return Money(self._competitiveness, self._competitiveness_currency)

    @competitiveness.setter
    def competitiveness(self, value):
        self._competitiveness = value.amount
        self._competitiveness_currency = value.currency

    @property
    def fundamental_value(self):
        if not self._fundamental_value:
            return None
        return Money(self._fundamental_value, self._fundamental_value_currency)

    @fundamental_value.setter
    def fundamental_value(self, value):
        self._fundamental_value = value.amount
        self._fundamental_value_currency = value.currency

    @property
    def spread(self):
        if not self._spread:
            return None
        return Money(self._spread, self._spread_currency)

    @spread.setter
    def spread(self, value):
        self._spread = value.amount
        self._spread_currency = value.currency
    
    @property
    def volume_filled(self):
        return sum([t.volume for t in self.trades], Money("0", self._volume_currency))

    @property
    def volume_remaining(self):
        return self.volume - self.volume_filled

    # Included for backwards compatibility. New code should call calc_position()
    @property
    def position(self):
        return self.calc_position()

    def calc_position(self, include_fees=True):
        from gryphon.lib.models.exchange import Position
        return sum([t.calc_position(include_fees=include_fees) for t in self.trades], Position())
